import networkx as nx

from pamogk.lib.sutils import *


# options are raise, warn, print (default)
# np.seterr(all='raise')

def kernel(pat_ids, pathway, label_key, alpha=0.5, epsilon=1e-6, normalization=False):
    """
    Parameters
    ----------
    pat_ids:
        list of patient ids
    pathway:
        pathway networkx graph
    alpha: float
        the smoothing parameter
    label_key: str
        label attribute key showing patient to label mapping. Should be filled in experiment
    epsilon: {1e-6} float
        smoothing converges if the change is lower than epsilon
    normalization: {False} bool
        normalize the kernel matrix such that the diagonal is 1
    """

    num_pat = pat_ids.shape[0]
    pat_ind = {}
    for ind, pid in enumerate(pat_ids): pat_ind[pid] = ind
    # extract labels of nodes of graphs
    mutations = np.zeros([num_pat, len(pathway.nodes)], dtype=np.float)
    for nid in pathway.nodes:
        nd = pathway.nodes[nid]
        for pid, lb in nd[label_key].items():
            if pid in pat_ind.keys():
                try:
                    _lb = float(lb)
                except ValueError:
                    _lb = 1
                mutations[pat_ind[pid], nid] = _lb

    # extract the adjacency matrix on the order of nodes we have
    adj_mat = nx.to_numpy_array(pathway, nodelist=pathway.nodes)
    ordered_graph = nx.OrderedGraph()
    ordered_graph.add_nodes_from(pathway.nodes())
    ordered_graph.add_edges_from(sorted(list(pathway.edges())))

    # smooth the mutations through the pathway
    mutations = smooth(mutations, adj_mat, alpha, epsilon)
    # get all pairs shortest paths
    all_pairs_sp = nx.all_pairs_shortest_path(ordered_graph)

    km = np.zeros((num_pat, num_pat))

    checked = []
    for src, dsp in all_pairs_sp:  # iterate all pairs shortest paths
        # add source node to checked nodes so we won't check it again in destinations
        checked.append(src)
        # skip if the source is not gene/protein
        if pathway.nodes[src]['type'] != 'Protein': continue
        # otherwise
        for dst, sp in dsp.items():
            # if destination already checked skip
            if dst in checked: continue
            # if the destination is not gene/protein skip
            if pathway.nodes[sp[-1]]['type'] != 'Protein': continue
            ind = np.isin(pathway.nodes, sp)
            tmp_md = mutations[:, ind]
            # calculate similarities of patients based on the current pathway
            tmp_km = tmp_md @ np.transpose(tmp_md)
            km += tmp_km  # update the main kernel matrix

    # normalize the kernel matrix if normalization is true
    if normalization == True: km = normalize_kernel_matrix(km)

    return km


def smooth(md, adj_m, alpha=0.5, epsilon=10 ** -6):
    """
    md: numpy array
        a numpy array of genes of patients indicating which one is mutated or not
    adj_m: numpy array
        the adjacency matrix of the pathway
    alpha: {0.5} float
        the smoothing parameter in range of 0-1
    epsilon: {1e-6} float
        smoothing converges if the change is lower than epsilon
    """
    # since alpha will be together with norm_adj_mat all the time multiply here
    alpha_norm_adj_mat = alpha * adj_m / np.sum(adj_m, axis=0)

    s_md = md
    pre_s_md = md + epsilon + 1

    while np.linalg.norm(s_md - pre_s_md) > epsilon:
        pre_s_md = s_md
        # alpha_norm_adj_mat already includes alpha multiplier
        s_md = (s_md @ alpha_norm_adj_mat) + (1 - alpha) * md

    return s_md


def normalize_kernel_matrix(km):
    kmD = np.array(np.diag(km))
    kmD[kmD == 0] = 1
    D = np.diag(1 / np.sqrt(kmD))
    norm_km = np.linalg.multi_dot([D, km, D.T])  # K_ij / sqrt(K_ii * K_jj)
    return np.nan_to_num(norm_km)  # replace NaN with 0


def main():
    # Create a networkx graph object
    mg = nx.Graph()

    # Add edges to to the graph object
    # Each tuple represents an edge between two nodes
    mg.add_edges_from([
        ('A', 'B'),
        ('A', 'C'),
        ('C', 'D'),
        ('A', 'E'),
        ('C', 'E'),
        ('D', 'B'),
        ('B', 'C'),
        ('C', 'X')])

    mg2 = nx.Graph()
    mg2.add_edges_from([
        ('A', 'B'),
        ('A', 'C'),
        ('C', 'D'),
        ('A', 'E'),
        ('C', 'E'),
        ('D', 'B'),
        ('B', 'C'),
        ('C', 'X')])

    nx.set_node_attributes(mg, {'X': 0, 'A': 1, 'B': 0, 'C': 0, 'D': 1, 'E': 0}, 'label')
    # nx.set_node_attributes(mg, {'X':'Protein', 'A':'Calcium', 'B':'Protein', 'C':'Protein', 'D':'Calcium', 'E':'Protein'}, 'type')
    nx.set_node_attributes(mg, {'X': 'Protein', 'A': 'Protein', 'B': 'Protein', 'C': 'Protein', 'D': 'Protein',
                                'E': 'Protein'}, 'type')

    nx.set_node_attributes(mg2, {'X': 1, 'A': 1, 'B': 0, 'C': 0, 'D': 1, 'E': 0}, 'label')
    # nx.set_node_attributes(mg2, {'X':'Protein', 'A':'Calcium', 'B':'Protein', 'C':'Protein', 'D':'Calcium', 'E':'Protein'}, 'type')
    nx.set_node_attributes(mg2, {'X': 'Protein', 'A': 'Protein', 'B': 'Protein', 'C': 'Protein', 'D': 'Protein',
                                 'E': 'Protein'}, 'type')

    # smoothing parameter for the kernel
    alpha = 0.1

    # calculate the kernel using PAMOGK
    # NOTE: this might not be working after some changes to the kernel method
    km = kernel(np.array([0, 1]), mg, alpha=alpha, label_key='label')

    # display the resulting kernel matrix
    print('Kernel matrix calculated by PAMOGK with alpha', alpha)
    print(km)


if __name__ == '__main__':
    main()



import csv

import numpy as np
import pandas as pd

from .. import config

RPPA_DATA_DIR = config.DATA_DIR / 'rppa'


def prune_proteins(data):
    tmp = data['#probe'].str.split('-')
    prots = []
    un_proc_prots = []
    for prot in tmp:
        pieces = prot[:-2]

        un_proc_prot_name = '-'.join(pieces)
        tmp = ''.join(pieces)
        prot_name = ''.join(tmp.split('_'))
        prots.append(prot_name.upper())
        un_proc_prots.append(un_proc_prot_name)
    return prots, un_proc_prots


def read_csv(path, has_header=False, delimiter=',', mapper=lambda r: r):
    path = RPPA_DATA_DIR / path
    with open(path, 'r', encoding='utf8', errors='ignore') as file:
        csv_reader = csv.reader(file, delimiter=delimiter)
        if has_header:
            next(csv_reader)
        return [mapper(r) for r in csv_reader if len(r) > 1]


def get_entrez_data():
    # First mapping
    p2eg = read_csv('gene_uniprot_entrez_kegg.csv', has_header=True, delimiter=';', mapper=lambda r: [r[0], r[3]])

    # Second mapping
    add_map = read_csv('external_mapping_of_rppa_proteins.csv')

    # Third mapping
    ext_eg_id_map = read_csv('external_mapping_of_rppa_proteins_with_entrez_gene_ids.txt')

    # Fourth mapping
    manual_map = read_csv('manual_mapping.txt')

    return np.array(p2eg), np.array(add_map), np.array(ext_eg_id_map), np.array(manual_map)


def process(filename, is_continuous=False, threshold=1.96):
    """
    :param filename: file loc of synapse rppa data
    :param is_continuous: whether or not to discritize z-scores if True returns them as is otherwise discritize them to -1 and 1
    :param threshold: defaults to 1.96 two standard deviation
    :return: a dataframe indicating the normalized values of expressions of genes where
                genes with entrez gene id are on rows and patient ids are on columns
    """

    data = pd.read_csv(config.get_safe_data_file(filename), sep='\t')
    # data = data.set_index(['Gene Name', 'Entrez Gene ID'])
    # data = data.set_index('Entrez Gene ID')
    drop_cols = []
    for col in data.columns:
        if not (col == '#probe') and not ('01' in col.split('-')[3]):
            drop_cols.append(col)

    data.drop(columns=drop_cols)
    proc_prots, unproc_prots = prune_proteins(data)

    # mappings
    p2eg, add_map, ext_eg_id_map, manual_map = get_entrez_data()

    eg_ids = np.zeros(len(proc_prots))
    # non_eg_ids = np.zeros(len(procProts))

    # Try p2eg
    for i in range(len(data)):
        if proc_prots[i] in p2eg[:, 0]:
            idx = np.where(p2eg[:, 0] == proc_prots[i])
            eg_ids[i] = p2eg[:, 1].astype(int)[idx]

    def map_eg_id(src):
        for i in range(len(unproc_prots)):
            if eg_ids[i] == 0:
                if unproc_prots[i] in src[:, 0]:
                    idx = np.where(src[:, 0] == unproc_prots[i])
                    eg_ids[i] = src[:, 2][idx]

    # Try ext_eg_id_map
    map_eg_id(ext_eg_id_map)

    # Try manual_map
    map_eg_id(manual_map)

    # data.replace(list(data['#probe']), list(eg_ids))
    data['#probe'] = eg_ids.astype(int)

    # drop the genes which are not expressed more than half of the samples
    genes_to_drop = data.index[(data == 0).T.sum().values > (len(data.columns) / 2)]
    data = data.drop(genes_to_drop)

    data = data.sort_index(axis=1)
    data = data.sort_values('#probe', ascending=True)
    data = data.drop_duplicates(subset='#probe', keep='first')
    data['#probe'] = np.array(data['#probe']).astype('str')
    data = data.set_index(['#probe'])

    # calculate z-scores
    mean_exp = data.mean(axis=1, numeric_only=True)
    std_exp = data.std(axis=1, numeric_only=True)
    z_scores = data.subtract(mean_exp, axis=0)
    z_scores = z_scores.div(std_exp, axis=0)

    if is_continuous:
        # find normalized expressions of genes
        gene_expression = pd.DataFrame(z_scores, index=data.index, columns=data.columns)
    else:
        # find differentially expressed genes
        gene_expression = pd.DataFrame(0, index=data.index, columns=data.columns)
        # 1 for over-expressed, -1 for under-expressed
        gene_expression[z_scores > threshold] = 1
        gene_expression[z_scores < (-1 * threshold)] = -1

    return gene_expression



import pandas as pd

from .. import config


def process(filename, is_continuous=False, threshold=1.96):
    """
    Parameters
    ----------
    filename: :str
    tab separated file in which the first row contains gene name/entrez gene id combined
        and patient ids. The rest of the rows are the genes and their expressions on each patient.
    is_continuous: :bool if true returns normalized z scores
    threshold: :float 1.96
        defaults to two standard deviation thresholding for under/over expressed for discrete calculation

    Returns
    -------
    gene_expressions: if is_continuous is True:
        a dataframe indicating the normalized values of expressions of genes where
        genes with entrez gene id are on rows and patient ids are on columns
        otherwise: a dataframe indicating the over- (1) and under-expressed (-1) genes where
        genes with entrez gene id are on rows and patient ids are on columns

    gene_name_map: a dataframe indicating the name of the genes of entrez gene ids
    """

    fpath = config.get_safe_data_file(filename)
    data = pd.read_csv(fpath, sep='\t')
    # data = data.set_index(['Gene Name', 'Entrez Gene ID'])
    # data = data.set_index('Entrez Gene ID')
    data[['gene_name', 'entrez_gene_id']] = data['#probe'].str.split('|', n=1, expand=True)
    data = data.set_index(['entrez_gene_id'])
    data = data.drop(columns=['#probe'])

    # IF THERE ARE ADDITIONAL PROBLEMS TO BE CONSIDERED, PREPROCESS THE DATA ACCORDINGLY
    # drop genes with name '?'
    tmp = data.index[data['gene_name'] == '?']
    data = data.drop(tmp)

    # sort the dataframe based on both gene name and patient id
    data = data.sort_index(axis=1)
    data = data.sort_values('gene_name')

    gene_name_map = data['gene_name']
    data = data.drop(columns=['gene_name'])

    # delete patients with non-solid tumor
    tmp = [row[3][0:2] == '01' for row in data.columns.str.split('-').tolist()]
    ind = [i for i, x in enumerate(tmp) if x == False]  # find the indices of non-solid tumors
    data = data.drop(columns=data.columns[ind])  # drop them from the data frame

    # drop the genes which are not expressed more than half of the samples
    genes_to_drop = data.index[(data == 0).T.sum().values > (len(data.columns) / 2)]
    data = data.drop(genes_to_drop)

    # calculate z-scores
    mean_exp = data.mean(axis=1, numeric_only=True)
    std_exp = data.std(axis=1, numeric_only=True)
    z_scores = data.subtract(mean_exp, axis=0)
    z_scores = z_scores.div(std_exp, axis=0)

    if is_continuous:
        gene_expression = pd.DataFrame(z_scores, index=data.index, columns=data.columns)
    else:
        # find differentially expressed genes
        gene_expression = pd.DataFrame(0, index=data.index, columns=data.columns)
        # 1 for over-expressed, -1 for under-expressed
        gene_expression[z_scores > threshold] = 1
        gene_expression[z_scores < -threshold] = -1

    return gene_expression, gene_name_map



from .. import config
import heapq
import mosek
import sys

import numpy as np
import numpy.matlib as npML
from sklearn.cluster import KMeans


def lmkkmeans_train(Km, iteration_count=2, cluster_count=10):
    if iteration_count < 1:
        raise ValueError(f'iteration_count cannot be non positive give: {iteration_count}')
    N = Km.shape[1]
    P = Km.shape[0]
    Theta = np.zeros((N, P))
    for i in range(N):
        for j in range(P):
            Theta[i][j] = 1 / P
    K_Theta = calculate_localized_kernel_theta(Km, Theta)
    objective = np.zeros((iteration_count, 1))

    H = None
    for it in range(iteration_count):
        eig, H = np.linalg.eig(K_Theta)
        maxIndexes = heapq.nlargest(cluster_count, range(eig.shape[0]), eig.take)
        H = H[:, maxIndexes]
        H_con = np.conjugate(H.transpose())
        HHT = H @ H_con

        qsubi = []
        qsubj = []
        qval = []
        for m in range(P):
            start_index = m * N
            tmpMatrix = np.eye(N) * Km[m] - HHT * Km[m]
            for i in range(len(tmpMatrix)):
                for j in range(i + 1):
                    if tmpMatrix[i, j] != 0:
                        qsubi.append(start_index + i)
                        qsubj.append(start_index + j)
                        qval.append(tmpMatrix[i, j])
        resxx = call_mosek(qsubi, qsubj, qval, N, P)
        Theta = np.array(resxx).reshape((P, N)).T
        K_Theta = calculate_localized_kernel_theta(Km, Theta)
        objective[it] = np.trace(H_con @ K_Theta @ H) - np.trace(K_Theta)
        print()

    if H is None:
        raise ValueError('No iterations completed causing H to be invalid')
    # normalize vector lengths
    tempH = np.linalg.norm(H, axis=1, keepdims=True)
    tempH[tempH == 0] = 1
    H_normalized = H / tempH
    clustering = KMeans(n_clusters=cluster_count, max_iter=1000).fit_predict(H_normalized)
    return clustering, H_normalized


def calculate_localized_kernel_theta(K, Theta):
    N = K.shape[1]
    K_Theta = np.zeros((N, N))
    for i in range(K.shape[0]):
        K_Theta += Theta[:, i] * Theta[:, [i]] * K[i]
    return K_Theta


def call_mosek(qsubi, qsubj, qval, N, P):
    with mosek.Env() as env:
        # Attach a printer to the environment
        env.set_Stream(mosek.streamtype.log, streamprinter)
        # Create a task
        with env.Task() as task:
            task.set_Stream(mosek.streamtype.log, streamprinter)
            # Set up and input bounds and linear coefficients
            bkc = [mosek.boundkey.ra] * N
            blc = np.ones((N, 1))
            buc = np.ones((N, 1))
            bkx = [mosek.boundkey.ra] * N * P
            blx = np.zeros((N * P, 1))
            bux = np.ones((N * P, 1))
            c = np.zeros((N * P, 1))
            aval = np.ones((N * P, 1)).astype(int)
            asub = np.zeros((N * P, 1)).astype(int)
            for i in range(P):
                for j in range(N):
                    asub[j + N * i] = j

            numvar = len(bkx)
            numcon = len(bkc)
            # Append 'numcon' empty constraints.
            # The constraints will initially have no bounds.
            task.appendcons(numcon)
            # Append 'numvar' variables.
            # The variables will initially be fixed at zero (x=0).
            task.appendvars(numvar)
            for j in range(numvar):
                # Set the linear term c_j in the objective.
                task.putcj(j, c[j])
                # Set the bounds on variable j
                # blx[j] <= x_j <= bux[j]
                task.putvarbound(j, bkx[j], blx[j], bux[j])
                # Input column j of A
                task.putacol(j,  # Variable (column) index.
                             # Row index of non-zeros in column j.
                             asub[j],
                             aval[j])  # Non-zero Values of column j.
            for i in range(numcon):
                task.putconbound(i, bkc[i], blc[i], buc[i])
            # Set up and input quadratic objective
            task.putqobj(qsubi, qsubj, qval)
            # Input the objective sense (minimize/maximize)
            task.putobjsense(mosek.objsense.minimize)
            # Optimize
            task.optimize()
            # Print a summary containing information
            # about the solution for debugging purposes
            task.solutionsummary(mosek.streamtype.msg)
            # prosta = task.getprosta(mosek.soltype.itr)
            # solsta = task.getsolsta(mosek.soltype.itr)
            # Output a solution
            xx = [0.] * numvar
            task.getxx(mosek.soltype.itr, xx)
            return xx


def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()


def main():
    try:
        print(config.MOSEK_LICENCE_FILE_PATH)
        # TODO generate some example kernels
        v1, v2, v3 = views.getLinearKernel()
        Kmm = np.stack((v1, v2, v3))

        lmkkmeans_train(Kmm, cluster_count=3, iteration_count=10)
    except mosek.MosekException as e:
        print("ERROR: %s" % str(e.errno))
        if e.msg is not None:
            import traceback

            traceback.print_exc()
            print("\t%s" % e.msg)
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Label Evaluation
"""
import json

import matplotlib.pyplot as plt
import pandas as pd
from lifelines import KaplanMeierFitter
from lifelines.statistics import multivariate_logrank_test

from pamogk import config
from pamogk.lib.sutils import *
from pamogk.result_processor.latex_generator import pandas_to_latex_table

example_exp_path = config.DATA_DIR / 'pamogk_kirc' / 'pamogk_exp-Experiment1' / \
                   'label=th196-smoothing_alpha=0.9-kr_norm=True'
default_clinical_data_path = config.DATA_DIR / 'kirc_data/kirc_clinical_data.csv'

# create the parser with some defaults
parser = argparse.ArgumentParser(description='Evaluate labels')
parser.add_argument('--clinical-data', '-c', metavar='file-path', dest='clinical_data_path', type=str2path,
                    help='Clinical Data', default=default_clinical_data_path)
parser.add_argument('--show-conf-int', '-ci', dest='show_ci', action='store_true', help='Show Confidence Intervals')
parser.add_argument('--results-dir', '-r', metavar='directory', dest='results_dir', type=str2path,
                    help='Experiment Data Dir', default=example_exp_path)

args = None


class LabelAnalysis(object):
    def __init__(self, results_dir, show_ci=False, clinical_data_path=default_clinical_data_path, methods=None,
                 cluster_sizes=None, log2_lambdas=None):
        if log2_lambdas is None:
            log2_lambdas = list(range(-15, 16, 3))
        if cluster_sizes is None:
            cluster_sizes = list(range(2, 6))
        if methods is None:
            methods = ['mkkm', 'kmeans']
        print_args(locals())

        self.res_dir = Path(results_dir)
        self.exp_data_dir = self.res_dir.parent
        self.show_ci = show_ci
        self.fig_dir = self.res_dir / 'figures'
        self.exports_path = self.fig_dir / 'exports.json'
        self.log2_lambdas = log2_lambdas
        self.cluster_sizes = cluster_sizes
        self.methods = methods

        safe_create_dir(self.fig_dir)
        # load clinical and experiment data
        clinical_data = self.read_clinical_data(clinical_data_path)
        exp_pat_ids = self.load_exp_patients()
        # this array holds true for patients that have data for the patient at this index
        self.exp_pats_with_data = np.array([pat_id in clinical_data for pat_id in exp_pat_ids])
        # these are clinical data for patients with data
        pat_ids = exp_pat_ids[self.exp_pats_with_data]
        self.clinical_days = np.array([clinical_data[pat_id]['days'] for pat_id in pat_ids]).astype(int)
        self.clinical_status = np.array([clinical_data[pat_id]['status'] for pat_id in pat_ids]).astype(int)
        self.exported_files = {}

    @timeit
    def load_exp_patients(self):
        """
        Loads list of intersecting patients from exp data dir
        """
        with open(self.exp_data_dir / 'patients.csv') as f:
            reader = csv.reader(f)
            return np.array([row[0] for row in reader])

    @timeit
    def read_label_data(self, filename):
        """
        Reads the label data and patient_id data and create array of dict with elements pat_id and label
        Parameters
        ----------
        filename

        Returns
        -------

        """
        return np_load_data(self.res_dir / f'{filename}.npz', key='labels')

    @timeit
    def read_clinical_data(self, file_loc):
        """
        Load read clinical data
        Parameters
        ----------
        file_loc

        Returns
        -------

        """
        clinical_data = {}
        with open(file_loc) as f:
            reader = csv.DictReader(f)
            for row in reader:
                clinical_data[row['Patient ID']] = dict(status=row['Status'], days=row['Days'])
        return clinical_data

    def calc_logrank_p_value(self, labels):
        """
        Calculates p value for given labels using clinical data
        Parameters
        ----------
        labels: iterable labels, must have same size as self.exp_pats_with_data

        Returns
        -------

        """
        return multivariate_logrank_test(self.clinical_days, labels, self.clinical_status).p_value

    @timeit
    def km_analysis(self, labels, out_filename):
        """
        Parameters
        ----------
        labels:
            Array patient dicts, which includes pat_id, label, status, days
        out_filename:
            Name of file to save Kaplan Meier Graph

        Return
        -----------
        [0]:
            Multivariate log-rank test p values
        [1]:
            One-vs-All result of log-rank test (array of p values)
        """
        plt.clf()

        ax = plt.subplot(111)
        ax.set_xlabel('Time (day)')
        ax.set_ylabel('Survival Probability')
        for label in np.unique(labels):  # get unique cluster labels
            cluster_pat_ind = labels == label  # store indices of patients in this cluster
            days = self.clinical_days[cluster_pat_ind]
            status = self.clinical_status[cluster_pat_ind]
            kmf = KaplanMeierFitter()
            kmf.fit(days, status, label=f'size={np.sum(cluster_pat_ind)}')
            # save to be used by others
            out_path = self.fig_dir / f'{out_filename}-label={label}.npz'
            np_save_npz(out_path, kmf_prob_func=kmf.survival_function_.values,
                        kmf_conf_int=kmf.confidence_interval_.values, kmf_timeline=kmf.timeline)
            self.add_exported_filepath(out_path)
            kmf.plot(ax=ax, ci_show=self.show_ci)

        out_path = self.fig_dir / f'{out_filename}.png'
        plt.savefig(out_path)
        self.add_exported_filepath(out_path)

        # all
        all_p = self.calc_logrank_p_value(labels)
        # one vs all
        vs_p = [f"{self.calc_logrank_p_value(labels == lb):.2e}" for lb in np.unique(labels)]

        return all_p, vs_p

    # Process one file
    def process_label_file(self, filename):
        """
        Takes label from label_file and saves figure to out_file location
        Returns the multivariate p result and one-vs-all p result.

        Parameters
        ----------
        filename: str
            Filename that shows the path for the kernel

        Return
        -----------
        [0]:
            Multivariate log-rank test p values
        [1]:
            One-vs-All result of log-rank test (array of p values)
        """
        labels = np_load_data(self.res_dir / f'{filename}.npz', key='labels')
        # filter out patients that have no data
        labels = labels[self.exp_pats_with_data]

        p, vs_p = self.km_analysis(labels, filename)

        return p, vs_p

    def add_exported_filepath(self, path):
        path = Path(path).absolute()
        if not path.exists():
            ValueError('Tried to add non existing path to exports')
        if path.suffix not in self.exported_files:
            self.exported_files[path.suffix] = []
        self.exported_files[path.suffix].append(str(path))

    def df_to_csv(self, df, name):
        out_path = (self.fig_dir / name).with_suffix('.csv')
        df.to_csv(out_path)
        self.add_exported_filepath(out_path)

    def df_to_latex_table(self, df, name, row_name='l', col_name='k'):
        out_path = (self.fig_dir / name).with_suffix('.tex')
        pandas_to_latex_table(df, row_name, col_name, out_path)
        self.add_exported_filepath(out_path)

    @timeit
    def run(self):
        log2_lmbds_str = [str(lmbd) for lmbd in self.log2_lambdas]
        result_inds = log2_lmbds_str.copy()

        for method in self.methods:
            if method != 'mkkm':
                result_inds.append(method)

        result_df = pd.DataFrame(columns=self.cluster_sizes, index=result_inds)

        for method in self.methods:
            for label in self.cluster_sizes:
                # compare dataframe
                comp_df = pd.DataFrame(columns=list(range(1, label+1)), index=log2_lmbds_str)
                if method == 'mkkm':
                    for (log2_lmbd, log2_lmbd_str) in zip(self.log2_lambdas, log2_lmbds_str):
                        res, vs_res = self.process_label_file(f'pamogk-{method}-k={label}-log2_lambda={log2_lmbd}')
                        result_df.loc[log2_lmbd_str][label] = f'{res:.2e}'
                        comp_df.loc[log2_lmbd_str] = vs_res

                    self.df_to_latex_table(comp_df, f'logrank-test-k={label}.tex')
                    self.df_to_csv(comp_df, f'logrank-test-k={label}')
                else:
                    res, vs_res = self.process_label_file(f'pamogk-{method}-k={label}')
                    result_df.loc[method][label] = f'{res:.2e}'

        self.df_to_latex_table(result_df, 'latex_table')
        self.df_to_csv(result_df, 'multivariate-logrank-test')
        with open(self.exports_path, 'w') as f:
            json.dump(self.exported_files, f)
        log(f'Finished label analysis with exported files on path={self.exports_path}')


if __name__ == '__main__':
    # if running directly use command line arguments
    args = parser.parse_args()
    LabelAnalysis(**vars(args)).run()



"""
for pw in all_pathways:
    for n in pw:
        rw = random walks x RW_CNT
        vis_freq_map = freq_count_of_genes(rw)
        neigh_w[n] = calculate_weigted_neighbor_list(vis_freq_map)

for p in patients:
    for m in mutate_genes[p]:
        for n in neighbors[m]:
            w[n] += neigh_w[m][n]
"""


import argparse
import csv

from gensim.models.keyedvectors import KeyedVectors

from .. import config
from ..gene_mapper import uniprot_mapper as um
from ..pathway_reader import cx_pathway_reader

'''
Calling get_n2v_representations() returns vectors for each patient.
'''

KIRC_PATH = config.DATA_DIR / 'kirc_data' / 'kirc_somatic_mutation_data.csv'
UNIPROT_ENTREZ_MAP_FPATH = config.ROOT_DIR / 'gene_mapper' / 'uniprot-entrez-map.tsv'


def patient_entrez_to_uniprot():
    list_of_gene_patient = []
    with open(KIRC_PATH, 'r') as csvfile:
        csv_reader = csv.reader(csvfile, delimiter=',', quotechar='|')
        next(csv_reader)
        for row in csv_reader:
            if int(row[1]) != 0:
                list_of_gene_patient.append(list(row))

    u_map = []
    e_map = []
    with open(UNIPROT_ENTREZ_MAP_FPATH, 'r') as csvfile:
        csv_reader = csv.reader(csvfile, delimiter='\t', quotechar='|')
        next(csv_reader)  # skip header
        for row in csv_reader:
            u_map.append(row[0])
            e_map.append(row[1])

    uni_to_entrez, entrez_to_uni = um.json_to_dict()

    patient_uniprot_list = []
    for row in list_of_gene_patient:
        uni_prot = []
        patient = row[2]

        try:
            uni_prot.append(entrez_to_uni[row[1]])
        except:
            if row[2] in e_map:
                uni_prot.append(u_map[e_map.index(row[2])])
            else:
                print("none")

        if len(uni_prot) != 0:
            patient_uniprot_list.append([patient, uni_prot])
    return patient_uniprot_list


def find_pathways_for_all_patients():
    patient_uniprot_list = patient_entrez_to_uniprot()
    count = 0
    pat_pws = []
    pathways = cx_pathway_reader.get_pathway_map()
    for patient in patient_uniprot_list:
        pathways_part = []
        count += 1
        if len(patient[1]) > 0:
            for geneList in patient[1]:
                x = cx_pathway_reader.get_pathways_with_genes(pathways, geneList)
                pathways_part.append(x)
        if len(pathways_part) != 0:
            pat_pws.append([patient[0], pathways_part])

    path = config.DATA_DIR / 'KircPathways.txt'
    with open(path, 'w') as f:
        for pat_pw in pat_pws:
            print(','.join(pat_pw[1]), file=f)


def get_patient_pathways_from_file():
    patient_pathway_list = []
    path = config.DATA_DIR / 'KircPathways.txt'
    with open(path, 'r') as f:
        for line in f:
            line = line[:-1]
            tokens = line.split(",")
            patient_pathway_list.append([tokens[0], tokens[1:]])
    return patient_pathway_list


def get_patient_uniprots_from_file():
    patient_uniprot_list = []
    path = config.DATA_DIR / 'KircUniprots.txt'
    with open(path, 'r') as f:
        for line in f:
            line = line[:-1]
            tokens = line.split(',')
            patient_uniprot_list.append([tokens[0], tokens[1:]])
    return patient_uniprot_list


def get_n2v_representations():
    parser = argparse.ArgumentParser(description='Run SPK algorithms on pathways')
    parser.add_argument('--pathways', metavar='pathway-id', type=str, nargs='+', help='pathway ID list',
                        default=['hsa04151'])
    parser.add_argument('--debug', action='store_true', dest='debug', help='Enable Debug Mode')
    parser.add_argument('--node2vec-p', '-p', metavar='p', dest='p', type=float, help='Node2Vec p value', default=1)
    parser.add_argument('--node2vec-q', '-q', metavar='q', dest='q', type=float, help='Node2Vec q value', default=1)
    parser.add_argument('--node2vec-size', '-n', metavar='node2vec-size', dest='n2v_size', type=float,
                        help='Node2Vec feature space size', default=128)
    parser.add_argument('--run-id', '-r', metavar='run-id', dest='rid', type=str, help='Run ID', default=None)
    parser.add_argument('--directed', '-d', dest='is_directed', action='store_true', help='Is graph directed',
                        default=False)
    parser.add_argument('--num-pat', dest='num_pat', type=int, help='Number of Patients for Synthetic Experiments',
                        default=1000)
    parser.add_argument('--surv-dist', '-s', dest='surv_dist', type=float,
                        help='Surviving patient percentage in range [0, 1]', default=0.9)
    parser.add_argument('--mut-dist', '-m', dest='mut_dist', type=float, help='Mutated gene percentage in range [0, 1]',
                        default=0.4)

    args = parser.parse_args()
    patient_uniprot_list = get_patient_uniprots_from_file()
    patient_pathway_list = get_patient_pathways_from_file()
    count = 0
    patient_vector_list = []
    for patient_pathway in patient_pathway_list:
        vector_list = []
        for pathway in patient_pathway[1]:
            nx_g = cx_pathway_reader.read_single_pathway(pathway)
            # gene_vec_map = node2vec_processor.process(pathway, nx_g, args)
            filename = f'{pathway}-p={args.p:0.2f}-q={args.q:0.2f}-dir={args.is_directed}-run={args.rid}-word2vec.csv'
            gene_vec_map = {}
            filepath = config.DATA_DIR / 'node2vec' / filename
            key_vectors = KeyedVectors.load_word2vec_format(filepath, binary=False)
            for (eid, gene_vec) in zip(key_vectors.index2entity, key_vectors.vectors):
                gene_vec_map[int(eid)] = gene_vec
            uniprot_list = patient_uniprot_list[count][1]
            for gene in list(nx_g.nodes(data=True)):
                for alias in gene[1]['alias']:
                    if len(alias.split(':')) > 1:
                        alias = alias.split(':')[1]
                    if alias in uniprot_list:
                        vector_list.append(gene_vec_map[gene[0]])
        patient_vector_list.append([patient_pathway[0], vector_list])
        count += 1

    path = config.DATA_DIR / 'KircVectors.txt'
    with open(path, 'w') as f:
        for patient in patient_vector_list:
            line = patient[0]
            for vector in patient[1]:
                line += ";" + ','.join(map(str, vector))
            f.write(line + "\n")
    return patient_vector_list

# get_n2v_representations()
# find_pathways_for_all_patients()



import argparse
import csv
from pathlib import Path

import numpy as np

from .. import config
from ..lib.sutils import ensure_file_dir

'''
    This file is to process som file which has type of .maf
    It is enough to call processOneCancerSomatic with file location as argument to function:
        import synapse_som_processor as ssp
        ssp.processOneCancerSomatic('folder/fileName.maf')
        This returns 2d numpy array with columns GeneName, EntrezId, Patient Id

    To write the report to a file:
        import synapse_som_processor as ssp
        output = ssp.processOneCancerSomatic('folder/fileName.maf')
        ssp.writeToFile(output, fileLocation)
'''

# CancerTypes = ['BLCA','BRCA','COAD','GBM','HNSC','KIRC','LAML','LUAD','LUSC','OV','READ','UCEC']
CANCER_TYPES = ['BLCA', 'COAD', 'GBM', 'HNSC', 'LAML', 'LUAD', 'LUSC', 'OV', 'READ', 'UCEC']
parser = argparse.ArgumentParser(description='Run SPK algorithms on pathways')
parser.add_argument('--somatic-data', '-r', metavar='file-path', dest='somatic_data', type=str2path, help='Somatic Data',
                    default=config.DATA_DIR / 'kirc_data/kirc_somatic_mutation_data.csv')

args = parser.parse_args()


def process_one_cancer_somatic(filepath, delimiter='\t', start_row=1):
    data_array = []

    with open(filepath, 'r', encoding='utf8', errors='ignore') as file:
        csv_reader = csv.reader(file, delimiter=delimiter)
        for i in range(start_row):
            next(csv_reader)
        for row in csv_reader:
            if len(row) > 1:
                temp_list = [row[0]] + [row[1]] + [row[15]]
                data_array.append(temp_list)

    output = np.array(data_array)
    prune_ind = np.zeros(len(output))
    prune_list = []
    for idx, data in enumerate(output):
        tmp = data[2]
        splitted = tmp.split('-')
        if '01' in splitted[3]:
            prune_ind[idx] = 1
            data[2] = '-'.join(splitted[0:3])
        else:
            prune_list.append(idx)

    output = np.delete(output, prune_list, axis=0)

    return np.sort(output, axis=0)


# 'kirc/data/synapse_kirc_som_data.csv'
def write_to_file(report, filepath):
    with open(filepath, 'w', newline='') as csv_file:
        csv_writer = csv.writer(csv_file, delimiter=',', quotechar='|', quoting=csv.QUOTE_MINIMAL)
        csv_writer.writerow(['Gene Name', 'Entrez Gene ID', 'Patient ID'])
        csv_writer.writerows(report)


def print_report(report):
    print(len(report))
    print(len(list(set(report[:, 0]))))
    print(len(list(set(report[:, 2]))))


def report_all_cancer_types(data_dir):
    for cancer in CANCER_TYPES:
        filepath = data_dir / cancer / 'som.maf'
        report = process_one_cancer_somatic(filepath)
        print(cancer + '# of row, # of Unique Gene, #of Unique Patient')
        print_report(report)


def read_processed_data():
    # Real Data #
    # process RNA-seq expression data
    patients = {}
    with open(args.somatic_data) as csvfile:
        reader = csv.DictReader(csvfile)
        for row in reader:
            pat_id = row['Patient ID']
            ent_id = row['Entrez Gene ID']
            if pat_id not in patients:
                patients[pat_id] = {ent_id}
            else:
                patients[pat_id].add(ent_id)

    return patients

def process_and_save_cancer(cancer_type='BRCA', data_dir=Path('/home/yitepeli/ForExp/')):
    ct_lower = cancer_type.lower()
    outfile_path = data_dir / f'{ct_lower}_data' / f'{ct_lower}_somatic_mutation_data.csv'
    ensure_file_dir(outfile_path)
    filepath = data_dir / cancer_type / 'som.maf'
    rep = process_one_cancer_somatic(filepath)
    write_to_file(rep, outfile_path)


def process_and_save_all():
    for ct in CANCER_TYPES:
        process_and_save_cancer(ct)


# patient_dict = read_processed_data()
# draw_hist(patient_dict)
# report_all_cancer_types()
process_and_save_all()
print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import collections as coll
import json
import pdb

import networkx as nx
import requests

from .. import config
from ..lib.sutils import *

HOST = 'http://www.ndexbio.org/v2'
NCI_USER_ID = '301a91c6-a37b-11e4-bda0-000c29202374'

DATA_ROOT = config.DATA_DIR / 'cx'
safe_create_dir(DATA_ROOT)


def get_pathway_map():
    PATHWAY_LIST_PATH = DATA_ROOT / 'nci_pathway_list.json'

    if not PATHWAY_LIST_PATH.exists():
        try:
            url = f'{HOST}/user/{NCI_USER_ID}/showcase'
            log('Pathway map not found fetching from', url)
            r = requests.get(url)
            if r.status_code != 200:
                raise Exception(f'Failed to get pathway list reason={r.reason}')
        except requests.exceptions.ConnectionError as e:
            log(f'Could not connect to {HOST} server.')
            log(f'Please make sure you are connected to a network authorized to access {HOST}')
            raise Exception(f'Failed to connect to host={HOST}', e)

        with open(PATHWAY_LIST_PATH, 'w') as f:
            f.write(r.text)

    pathway_list = json.load(open(PATHWAY_LIST_PATH))

    pathway_map = coll.OrderedDict()
    for p in pathway_list:
        pathway_map[p['externalId']] = p
    return pathway_map


def _get_pathway_child(pathway_data, key):
    for d in pathway_data:
        if key in d:
            return d[key]
    return None


@timeit
def read_pathways():
    pathway_map = get_pathway_map()
    pw_map = coll.OrderedDict()
    pw_ids = pathway_map.keys()
    log(f'Pathway data_dir={DATA_ROOT}')
    for (ind, pw_id) in enumerate(pw_ids):
        log(f'Processing pathway {ind + 1:3}/{len(pw_ids)}', end='\t')
        pw_data = read_single_pathway(pw_id, reading_all=True)
        pw_map[pw_id] = pw_data
    log()
    return pw_map


def read_single_pathway(pathway_id, reading_all=False):
    pend = '\r' if reading_all else '\n'
    pathway_map = get_pathway_map()
    if pathway_id not in pathway_map:
        raise Exception('Pathway not found in pathway list')

    PATHWAY_PATH = DATA_ROOT / f'{pathway_id}.cx'

    if not PATHWAY_PATH.exists():
        url = f'{HOST}/network/{pathway_id}'
        log(f'Pathway with pathway_id={pathway_id} not found fetching from url={url}', end=pend, ts=not reading_all)
        r = requests.get(url)
        if r.status_code != 200:
            raise Exception(f'Failed to get pathway: {r.status_code}-{r.reason}')

        with open(PATHWAY_PATH, 'w') as f:
            f.write(r.text)
    else:
        log(f'Pathway with pathway_id={pathway_id} retrieved from local data dir', end=pend, ts=not reading_all)

    pathway_data = json.load(open(PATHWAY_PATH))

    G = nx.Graph()  # initialize empty graph

    # get node map
    node_list = _get_pathway_child(pathway_data, 'nodes')
    nodes = {}
    for n in node_list:
        nodes[n['@id']] = n

    # load node attributes
    node_attributes = _get_pathway_child(pathway_data, 'nodeAttributes')
    attr_dict = {}
    for attr in node_attributes:
        nid = attr['po']
        if nid not in attr_dict:
            attr_dict[nid] = {}
        attr_dict[nid][attr['n']] = attr['v']

    # load cartesian coordinates
    coords = _get_pathway_child(pathway_data, 'cartesianLayout')
    for coord in coords:
        nid = coord['node']
        if nid not in attr_dict:
            attr_dict[nid] = {}
        attr_dict[nid]['x'] = coord['x']
        attr_dict[nid]['y'] = coord['y']

    # add nodes to graph
    # NOTE networkx graphs only allow alphanumeric characters as attribute names no - or _
    for nid, n in nodes.items():
        attrs = attr_dict[nid]
        attrs['n'] = n['n']
        if 'r' in n:
            if 'alias' not in attrs:
                attrs['alias'] = [n['r']]
            else:
                attrs['alias'].append(n['r'])
        if 'alias' in attrs:  # create attribute for ids only
            tmp = [a.split(':') for a in attrs['alias']]
            attrs['uniprotids'] = [s[1] for s in tmp if len(s) > 1 and len(s[1]) > 0]
        G.add_node(nid, **attrs)

    # get edge map
    edge_list = _get_pathway_child(pathway_data, 'edges')
    edges = {}
    for e in edge_list:
        edges[e['@id']] = e

    # load edge attributes
    edge_attributes = _get_pathway_child(pathway_data, 'edgeAttributes')
    attr_dict = {}
    for attr in edge_attributes:
        eid = attr['po']
        if eid not in attr_dict:
            attr_dict[eid] = {}
        attr_dict[eid][attr['n']] = attr['v']
    # add edges to graph
    for eid in edges:
        e = edges[eid]
        attrs = attr_dict[eid]
        attrs['i'] = e['i']
        G.add_edge(e['s'], e['t'], **attrs)

    return G


def get_pathways_with_genes(pathways, genes):
    acceptedPathways = []
    for p in pathways:
        PATHWAY_PATH = config.DATA_DIR / f'{p}.cx'
        pathway_data = json.load(open(PATHWAY_PATH))
        nodesInPathway = _get_pathway_child(pathway_data, 'nodes')
        for node in nodesInPathway:
            uniprotId = node["r"]
            if len(uniprotId.split(":")) > 1:
                uniprotId = uniprotId.split(":")[1]
            if uniprotId in genes:
                acceptedPathways.append(p)
                break

    return acceptedPathways


def get_all_pws_paradigm():
    """
    Reads all cx pathways and converts them to PARADIGM pathway format
    """
    nmapping = {
        'protein': 'protein',
        'none': 'abstract',
        'smallmolecule': 'chemical',
    }
    emapping = {
        'controls-state-change-of': '-a>',
        'controls-phosphorylation-of': '-a>',
        'used-to-produce': '-a>',
        'consumption-controlled-by': '-a>',
        'controls-expression-of': '-t>',
        'neighbor-of': 'component>',
        'catalysis-precedes': '-a>',
        'controls-production-of': '-a>',
        'controls-transport-of': '-a>',
        'chemical-affects': 'component>',
        'reacts-with': 'component>',
        'in-complex-with': 'component>',
        'controls-transport-of-chemical': 'component>',
    }

    def cln(_n):
        _n = _n['alias'][0]
        if _n.startswith('uniprot knowledge'):
            return _n.split(':')[1]
        return _n.replace(' ', '_').replace(',', '_c_').replace(':', '_')

    pw_map = read_pathways()
    ntypes = set()
    etypes = set()
    nodes = []
    uids = set()
    edges = []
    for pw_id, pw in pw_map.items():
        print(f'Reading pathway={pw_id}')
        # pdb.set_trace()
        lines = []
        for n in pw.nodes().values():
            # print(n['n'], n['type'])
            ntypes.add(n['type'].lower())
            r = [nmapping[n['type'].lower()], cln(n)]
            lines.append(r)
            if r[1] not in uids:
                nodes.append(r)
                uids.add(r[1])
        for (f, t), e in pw.edges().items():
            etypes.add(e['i'].lower())
            r = [cln(pw.nodes[f]), cln(pw.nodes[t]), emapping[e['i']]]
            lines.append(r)
            edges.append(r)
        with open(f'../data/paradigm/{pw_id}.tsv', 'w') as f:
            f.write('\n'.join('\t'.join(r) for r in lines))
    with open('../../data/paradigm/pws.tsv', 'w') as f:
        f.write('\n'.join('\t'.join(r) for r in nodes) + '\n')
        f.write('\n'.join('\t'.join(r) for r in edges))


if __name__ == '__main__':
    get_all_pws_paradigm()
    # pw_map = read_pathways()
    pdb.set_trace()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import collections
import pdb

from .. import config
from ..data_processor import rnaseq_processor as rp, synapse_rppa_processor as rpp
from ..gene_mapper import uniprot_mapper
from ..lib.sutils import *

parser = argparse.ArgumentParser(description='Run PAMOGK-mut algorithms on pathways')
parser.add_argument('--rs-patient-data', '-rs', metavar='file-path', dest='rnaseq_patient_data', type=str2path,
                    help='rnaseq pathway ID list',
                    default=config.DATA_DIR / 'kirc_data/unc.edu_KIRC_IlluminaHiSeq_RNASeqV2.geneExp.whitelist_tumor.txt')
parser.add_argument('--rp-patient-data', '-rp', metavar='file-path', dest='rppa_patient_data', type=str2path,
                    help='rppa pathway ID list', default=config.DATA_DIR / 'kirc_data/kirc_rppa_data')
parser.add_argument('--som-patient-data', '-s', metavar='file-path', dest='som_patient_data', type=str2path,
                    help='som mut pathway ID list',
                    default=config.DATA_DIR / 'kirc_data/kirc_somatic_mutation_data.csv')
args = parser.parse_args()
print_args(args)


class ParadigmDataPrep(object):
    def __init__(self):
        self.exp_data_dir = config.DATA_DIR / 'paradigm'

        safe_create_dir(self.exp_data_dir)
        # change log and create log file
        change_log_path(self.exp_data_dir / 'run.log')
        log('exp_data_dir:', self.exp_data_dir)

    @timeit
    def read_rnaseq_data(self):
        # Real Data #
        # process RNA-seq expression data

        gene_exp, gene_name_map = rp.process(args.rnaseq_patient_data)

        # convert entrez gene id to uniprot id
        pat_ids = gene_exp.columns.values  # patient TCGA ids
        ent_ids = gene_exp.index.values  # gene entrez ids
        return gene_exp.values, pat_ids, ent_ids

    @timeit
    def read_rppa_data(self):
        # Real Data #
        # process RNA-seq expression data

        gene_exp = rpp.process(args.rppa_patient_data)

        # convert entrez gene id to uniprot id
        pat_ids = gene_exp.columns.values  # patient TCGA ids
        ent_ids = gene_exp.index.values  # gene entrez ids
        return gene_exp.values, pat_ids, ent_ids

    @timeit
    def read_som_data(self):
        # Real Data #
        # process RNA-seq expression data
        patients = {}
        with open(args.som_patient_data) as csvfile:
            reader = csv.DictReader(csvfile)
            for row in reader:
                pat_id = row['Patient ID']
                ent_id = row['Entrez Gene ID']
                if pat_id not in patients:
                    patients[pat_id] = {ent_id}
                else:
                    patients[pat_id].add(ent_id)
        patients = collections.OrderedDict(sorted(patients.items()))

        return patients

    @staticmethod
    def find_intersection_lists(list1, list2, list3):
        intersection_list = set(list1).intersection(list2, list3)
        return intersection_list

    @timeit
    def find_intersection_patients(self, rs_GE, rs_pat, rp_GE, rp_pat, som_pat):
        rs_pat_list = []
        for pat in rs_pat:
            new_id = '-'.join(pat.split('-')[0:3])
            rs_pat_list.append(new_id)

        rp_pat_list = []
        for pat in rp_pat:
            new_id = '-'.join(pat.split('-')[0:3])
            rp_pat_list.append(new_id)

        som_pat_list = []
        for pat in som_pat.keys():
            som_pat_list.append(pat)

        intersection_list = list(self.find_intersection_lists(rs_pat_list, rp_pat_list, som_pat_list))
        intersection_list.sort()
        intersect_loc = self.exp_data_dir / 'patients.csv'
        with open(intersect_loc, 'w') as f:
            kirc_int = list(intersection_list)
            writer = csv.writer(f)
            writer.writerow(kirc_int)

        rs_pat_deleted_list = []
        for idx, value in enumerate(rs_pat_list):
            if value not in intersection_list:
                rs_pat_deleted_list.append(idx)

        rs_pat = np.delete(rs_pat, rs_pat_deleted_list)
        rs_GE = np.delete(rs_GE, rs_pat_deleted_list, axis=1)

        rp_pat_deleted_list = []
        for idx, value in enumerate(rp_pat_list):
            if value not in intersection_list:
                rp_pat_deleted_list.append(idx)

        rp_pat = np.delete(rp_pat, rp_pat_deleted_list)
        rp_GE = np.delete(rp_GE, rp_pat_deleted_list, axis=1)

        som_pat_deleted_list = []
        for pat_id in som_pat.keys():
            if pat_id not in intersection_list:
                som_pat_deleted_list.append(pat_id)

        for item in som_pat_deleted_list:
            som_pat.pop(item, None)

        return rs_GE, rs_pat, rp_GE, rp_pat, som_pat

    @timeit
    def preprocess_seq_patient_data(self, GE, all_ent_ids):
        # get the dictionary of gene id mappers
        uni2ent, ent2uni = uniprot_mapper.json_to_dict()

        found_ent_ids = [eid in ent2uni for eid in all_ent_ids]
        ent_ids = np.array([eid for eid in all_ent_ids if eid in ent2uni])
        uni_ids = np.array([ent2uni[eid] for eid in ent_ids])

        log('uni_ids:', len(uni_ids))
        log('miss_ent_ids:', len(all_ent_ids) - sum(found_ent_ids))

        # prune genes whose uniprot id is not found
        GE = GE[found_ent_ids]
        return GE, uni_ids

    @timeit
    def preprocess_som_patient_data(self, patients):
        # get the dictionary of gene id mappers
        uni2ent, ent2uni = uniprot_mapper.json_to_dict()

        res = []
        num_empty = 0
        for pat_id, ent_ids in patients.items():
            # uni_ids = [uid for eid in ent_ids if eid in ent2uni for uid in ent2uni[eid]]
            uni_ids = [uid for eid in ent_ids if eid in ent2uni for uid in ent2uni[eid]]
            # if there are any matches map them
            '''
            if len(uni_ids) > 0: res.append({
                'pat_id': pat_id,
                'mutated_nodes': uni_ids,
            })
            else: num_empty += 1
            '''
            res.append({
                'pat_id': pat_id,
                'mutated_nodes': uni_ids,
            })
        log('removed patients:', num_empty)

        return res

    @timeit
    def save_data(self, fname, data, pat_ids, uni_ids):
        with (self.exp_data_dir / fname).open('w') as f:
            f.write('\t'.join(['gene_id', *[uids[0] for uids in uni_ids]]))
            for i, pid in enumerate(pat_ids):
                f.write('\n' + '\t'.join([str(v) for v in ['-'.join(pid.split('-')[:3]), *data[i][:]]]))


def main():
    exp = ParadigmDataPrep()

    # Patient part
    # RnaSeq Data
    rs_GE, rs_pat_ids, rs_ent_ids = exp.read_rnaseq_data()

    # Rppa Data
    rp_GE, rp_pat_ids, rp_ent_ids = exp.read_rppa_data()

    # Somatic mutation data
    som_patients = exp.read_som_data()

    # Find intersect
    rs_GE, rs_pat_ids, rp_GE, rp_pat_ids, som_patients = exp \
        .find_intersection_patients(rs_GE, rs_pat_ids, rp_GE, rp_pat_ids, som_patients)

    # Kernel part
    # RnaSeq Data
    rs_GE, rs_uni_ids = exp.preprocess_seq_patient_data(rs_GE, rs_ent_ids)
    exp.save_data('rnaseq.tsv', rs_GE.T, rs_pat_ids, rs_uni_ids)

    # Rppa Data
    rp_GE, rp_uni_ids = exp.preprocess_seq_patient_data(rp_GE, rp_ent_ids)

    # Somatic mutation data
    som_patients = exp.preprocess_som_patient_data(som_patients)

    pdb.set_trace()


if __name__ == '__main__':
    main()



import argparse

import pandas

from pamogk import config
from ..gene_mapper import uniprot_mapper
from ..kernels import node2vec_h_i_k as n2v
from ..lib.sutils import *
from ..pathway_reader import cx_pathway_reader as cx_pw

parser = argparse.ArgumentParser(description='Run SPK algorithms on pathways')
parser.add_argument('--patient-data', '-f', metavar='file-path', dest='patient_data', type=str2path, help='pathway ID list',
                    default=config.DATA_DIR / 'kirc_data/kirc_somatic_mutation_data.csv')

args = parser.parse_args()
print_args(args)

p, q = 1, 1
num_walks, walk_length = 10, 20
output = "xd.emb"


@timeit
def read_data():
    # Real Data #
    # process RNA-seq expression data
    patients = {}
    with open(args.patient_data) as csvfile:
        reader = csv.DictReader(csvfile)
        for row in reader:
            pat_id = row['Patient ID']
            ent_id = row['Entrez Gene ID']
            if pat_id not in patients:
                patients[pat_id] = {ent_id}
            else:
                patients[pat_id].add(ent_id)

    return patients


@timeit
def preprocess_patient_data(patients):
    # get the dictionary of gene id mappers
    uni2ent, ent2uni = uniprot_mapper.json_to_dict()

    res = []
    num_empty = 0
    for pat_id, ent_ids in patients.items():
        # uni_ids = [uid for eid in ent_ids if eid in ent2uni for uid in ent2uni[eid]]
        uni_ids = [uid for eid in ent_ids if eid in ent2uni for uid in ent2uni[eid]]
        # if there are any matches map them
        if len(uni_ids) > 0:
            res.append({
                'pat_id': pat_id,
                'mutated_nodes': uni_ids,
            })
        else:
            num_empty += 1

    log('removed patients:', num_empty)

    return res


def eliminate_with_conf(walk, conf, nodes):
    nodeFreq = np.zeros(len(nodes))
    len_walk = len(walk)
    threshold = conf * len_walk
    for w in walk:
        for nodeId in w:
            nodeFreq[nodeId] += 1

    return [idx for idx, freq in enumerate(nodeFreq) if freq >= threshold]


def process_walks(walks, conf, nodes):
    for node in nodes:
        walks[node] = eliminate_with_conf(walks[node], conf, nodes)


def get_neighbors_in_pathway(pw_graph, conf):
    nx_G = pw_graph

    id_gene_map = {}
    for nodeId in pw_graph._node.keys():
        id_gene_map[nodeId] = pw_graph._node[nodeId]["uniprotids"]

    directed = False
    G = n2v.Graph(nx_G, directed, p, q)
    G.preprocess_transition_probs()
    walks = G.simulate_walks(num_walks, walk_length)
    process_walks(walks, conf, walks.keys())
    # np.savetxt(output, walks)
    return walks, id_gene_map


@timeit
def read_pathways():
    # get all pathways
    return cx_pw.read_pathways()


@timeit
def get_neighbors_for_all_pathways(all_pw_map, conf):
    res = {}
    map = {}
    for ind, (pw_id, pw) in enumerate(all_pw_map.items()):
        neighbors, mapper = get_neighbors_in_pathway(pw, conf)
        res[pw_id] = neighbors
        map[pw_id] = mapper
        break
    return res, map


def calc_patientwise_score(neighbors, patient1, patient2, mapper):
    # from patient1 to patient2
    # find genes in graph and find unique set
    patient1_nodes = []
    for node in neighbors.keys():
        if len(set(mapper[node]).intersection(patient1["mutated_nodes"])) > 0:
            patient1_nodes.append(node)

    hit_count = 0
    for p1_node in patient1_nodes:
        for neigh in neighbors[p1_node]:
            if len(set(mapper[neigh]).intersection(patient2["mutated_nodes"])) > 0:
                hit_count += 1
                break
    if len(patient1_nodes) == 0:
        p1_rate = 0
    else:
        p1_rate = float(hit_count) / len(patient1_nodes)

    # from patient2 to patient1
    patient2_nodes = []
    for node in neighbors.keys():
        if len(set(mapper[node]).intersection(patient2["mutated_nodes"])) > 0:
            patient2_nodes.append(node)

    hit_count = 0
    for p2_node in patient2_nodes:
        for neigh in neighbors[p2_node]:
            if len(set(mapper[neigh]).intersection(patient1["mutated_nodes"])) > 0:
                hit_count += 1
                break

    if len(patient2_nodes) == 0:
        p2_rate = 0
    else:
        p2_rate = float(hit_count) / len(patient2_nodes)

    avg = (p1_rate + p2_rate) / 2

    return avg


@timeit
def calc_similarity_from_pathway(neighbors, patients, id_mapper):
    len_p = len(patients)
    similarityMatrix = np.zeros((len_p, len_p))
    for i in range(len_p):
        for j in range(i, len_p):
            score = calc_patientwise_score(neighbors, patients[i], patients[j], id_mapper)
            if i == j and score != 1:
                similarityMatrix[i, j] = 1
            else:
                similarityMatrix[i, j] = score
                similarityMatrix[j, i] = score

    return similarityMatrix


@timeit
def calc_kernel_from_similarity(similarityMatrix):
    distanceMatrix = 1 - similarityMatrix
    sigmasqList = [0.2, 0.5, 1, 5]
    sigmasq = sigmasqList[1]
    # it was scipt.exp but they should be doing the same thing
    return np.exp(-np.square(distanceMatrix) / (2 * sigmasq))


@timeit
def calc_kernel_from_pathways(neighbor_mappings, patients, id_mapper):
    kernel_res = None
    flag = 0
    len_p = len(patients)
    for pathway_id in neighbor_mappings.keys():
        similarityMatrix = calc_similarity_from_pathway(neighbor_mappings[pathway_id], patients, id_mapper[pathway_id])
        one_kernel = calc_kernel_from_similarity(similarityMatrix)
        if flag == 0:
            kernel_res = one_kernel
            kernel_res = np.reshape(kernel_res, (-1, len_p, len_p))
            flag = 1
        else:
            one_kernel = np.reshape(one_kernel, (-1, len_p, len_p))
            kernel_res = np.concatenate((kernel_res, one_kernel), axis=0)

    return kernel_res


def load_kernel(file_loc):
    data = pandas.read_csv(file_loc, delimiter=" ", dtype=np.float64)
    return np.array(data).reshape((165, 417, 417))


def isPSD(A, tol=1e-8):
    E, V = np.linalg.eigh(A)
    return np.all(E >= -tol)



import argparse
import csv
import time
from datetime import datetime
from pathlib import Path

import numpy as np

ISO_FORMAT = '%Y-%m-%dT%H:%M:%S.%f'


def timeit(f):
    """Timing decorator for functions. Just add @timeit to start and function
    will be timed. It will write starting and ending times

    Parameters
    ----------
    f : function
        decorator takes the function as parameter

    Returns
    -------
    mixed
        return value of function itself

    Raises
    ------
    Error
        when any error thrown by called function does not catch it
    """

    def wrapper(*args, **kwargs):
        log('Started:', f.__qualname__)
        t = time.time()
        res = f(*args, **kwargs)
        log(f'Finished: {f.__qualname__} elapsed: {time.time() - t:.2f}s')
        return res

    return wrapper


def safe_create_dir(d: Path):
    """
    Uses new pathlib
    Parameters
    ----------
    d: :obj:`pathlib.Path`
    """
    if not d.exists():
        log('Dir not found creating:', d)
        d.mkdir(parents=True)


def ensure_file_dir(file_path: Path):
    """
    Uses new pathlib
    Parameters
    ----------
    file_path: :obj:`pathlib.Path`
    """
    safe_create_dir(file_path.parent)


log_f = None
log_p = None


def change_log_path(path):
    global log_p, log_f
    if log_p == path:
        return
    if log_f:
        log_f.close()
    log_p = path
    ensure_file_dir(path)
    log_f = open(path, 'a')
    log('Initialized log_path:', path)


def logr(*args, **kwargs):
    log(*args, **kwargs, end='\r')


def log(*args, **kwargs):
    ts = datetime.now().strftime(ISO_FORMAT)[:-3]
    if 'ts' not in kwargs or kwargs['ts'] is not False:
        args = [ts, *args]
    if 'ts' in kwargs:
        del kwargs['ts']
    print(*args, **kwargs)
    if log_f:
        print(*args, **kwargs, file=log_f)
        log_f.flush()


def ensure_suffix(path, suffix='.npz'):
    if isinstance(path, Path):
        return path.with_suffix(suffix)
    if isinstance(path, str):
        if path[-len(suffix):] != suffix:
            return path + suffix
        return path
    raise TypeError('path must be either pathlib.Path or str')


def np_save_npz(path, data=None, **kwargs):
    if len(kwargs) == 0:
        if data is None:
            raise ValueError('Either data or kwargs must be given')
        kwargs = {'data': data}
    path = ensure_suffix(path)
    np.savez_compressed(path, **kwargs)


def np_load_data(path, key=None):
    """
    Loads numpy data from npz files
    Parameters
    ----------
    path
    key: str 'data'
    If given loads data in that key, if None is given loads all of the data in npz

    Returns
    -------

    """
    data = np.load(path)
    if key is not None:
        if key not in data:
            log(f'Given key={key} is not in loaded data on path={path}')
        return data[key]
    return data


def save_csv(path, rows):
    with open(path, 'w') as f:
        csvWriter = csv.writer(f)
        csvWriter.writerows(rows)


def print_args(args, ignore_keys=None):
    if ignore_keys is None:
        ignore_keys = ['self']
    arg_dict = args
    if type(args) is argparse.Namespace:
        arg_dict = vars(args)
    arg_dict = dict((k, v) for k, v in arg_dict.items() if k not in ignore_keys)
    m = max(len(k) for k in arg_dict.keys())
    log('Running args:')
    for k, v in arg_dict.items():
        log(f'  {k} {" " * (m - len(k))}: {v}')


def simplify_pat_ids(data):
    return ['-'.join(p.split('-')[:3]) for p in data]


def get_safe_path_obj(file_path):
    ftype = type(file_path)
    if not isinstance(file_path, Path):
        if ftype == str:
            return Path(file_path)
        else:
            raise TypeError(f'Type of file_path must be either str or pathlib.Path but given {ftype}')
    return file_path


def str2bool(v):
    """
    This is useful for the case of giving boolean parameters
    Parameters
    ----------
    v

    Returns
    -------

    """
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')


def str2path(v):
    """
    Just a wrapper for
    Parameters
    ----------
    v

    Returns
    -------

    """
    try:
        return Path(v)
    except TypeError:
        raise argparse.ArgumentTypeError('Invalid path provided')



import math

from sklearn.metrics.pairwise import linear_kernel
from sklearn.svm import SVC

from ..lib.sutils import *


def calculate_S_and_P(patients, gene_vectors, uni_to_vec):
    # calculate S (mutated gene vector set) and P (average mutataion point) vector
    kms_map = {}
    for p in patients:
        genes = []
        p['S'] = {}
        p['P'] = {}
        for pw_id, pw_genes in gene_vectors.items():
            for n in p['mutated_nodes']:
                genes.append(uni_to_vec[n])
            P = np.average(genes, axis=0)
            p['S'] = {pw_id: genes}
            p['P'] = {pw_id: P}
            if pw_id not in kms_map:
                kms_map[pw_id] = [P]
            else:
                kms_map[pw_id].append(P)
    # convert list to array
    for pw_id in kms_map:
        kms_map[pw_id] = np.vstack(kms_map[pw_id])
    return kms_map


def CP_kernels(kms, method=linear_kernel):
    return map(method, kms.values())


def test_accr(patients):
    hit = 0
    pids = [p['pid'] for p in patients]
    for pid in pids:
        test_p = [p for p in patients if p['pid'] == pid][0]
        train_p = [p for p in patients if p['pid'] != pid]

        linear_svc = SVC(kernel='linear')
        linear_svc.fit([p['P'] for p in train_p], [p['sick'] for p in train_p])
        is_hit = linear_svc.predict([test_p['P']]) == [test_p['sick']]
        # print('%3d: %s' % (pid, is_hit), linear_svc.predict([test_p['P']]), test_p['pid'], test_p['sick'])
        hit += is_hit[0]
    log('Accuracy Leave-One-Out accuracy=%.2lf' % (hit / len(pids)))

    hit = 0
    pids = [p['pid'] for p in patients]
    lpid = len(pids)
    K = 10
    S = math.ceil(lpid / K)
    for i in range(0, lpid, S):
        test_p = patients[i:i + S]
        train_p = patients[:i] + patients[i + S:]

        linear_svc = SVC(kernel='linear')
        linear_svc.fit([p['P'] for p in train_p], [p['sick'] for p in train_p])
        is_hit = linear_svc.predict([p['P'] for p in test_p]) == [p['sick'] for p in test_p]
        # print('%3d: %s' % (i, is_hit))
        hit += np.sum(is_hit)
    log('Accuracy K-fold with K=%d accuracy=%.2lf' % (K, hit / len(pids)))



import datetime
from pathlib import Path

import requests

from .sutils import safe_create_dir
from .. import config


def initialize_node2vec():
    LIB_DIR = Path(__file__).resolve().parent / 'lib'

    if not LIB_DIR.exists():
        print('libs_dir: creating')
        safe_create_dir(LIB_DIR)

    print('node2vec: checking')
    # load node2vec as python 3 module
    node2_vec_name = 'node2vec.py'
    node2_vec_path = config.LIB_DIR / node2_vec_name
    if not node2_vec_path.exists():
        r = requests.get('https://raw.githubusercontent.com/aditya-grover/node2vec/master/src/node2vec.py')
        with open(node2_vec_path, 'w') as q:
            for line in r.text.split('\n'):
                line = line.replace('save_word2vec_format', 'wv.save_word2vec_format')
                if 'print' in line:
                    line = line.replace('print', 'if self.debug: print(') + ')'
                    # gensim version update
                if 'def __init__' in line:
                    line = f'{line[:-2]}, debug=False):\n\t\tself.debug = debug'
                q.write(f'{line}\n')
            q.write(f'\n# accessed:{datetime.datetime.now()}')
        print('node2vec: installed')
    else:
        print('node2vec: already exists skipping')


initialize_node2vec()



"""
Reference implementation of node2vec.

Author: Aditya Grover

For more details, refer to the paper:
node2vec: Scalable Feature Learning for Networks
Aditya Grover and Jure Leskovec
Knowledge Discovery and Data Mining (KDD), 2016
"""

import argparse

import networkx as nx
import numpy as np
from gensim.models import Word2Vec

from ..kernels import node2vec_h_i as n2v


def parse_args():
    """
    Parses the node2vec arguments.
    """
    parser = argparse.ArgumentParser(description="Run node2vec.")

    parser.add_argument('--input', nargs='?', default='graph/karate.edgelist',
                        help='Input graph path')

    parser.add_argument('--output', nargs='?', default='emb/karate.emb',
                        help='Embeddings path')

    parser.add_argument('--dimensions', type=int, default=128,
                        help='Number of dimensions. Default is 128.')

    parser.add_argument('--walk-length', type=int, default=80,
                        help='Length of walk per source. Default is 80.')

    parser.add_argument('--num-walks', type=int, default=10,
                        help='Number of walks per source. Default is 10.')

    parser.add_argument('--window-size', type=int, default=10,
                        help='Context size for optimization. Default is 10.')

    parser.add_argument('--iter', default=1, type=int,
                        help='Number of epochs in SGD')

    parser.add_argument('--workers', type=int, default=8,
                        help='Number of parallel workers. Default is 8.')

    parser.add_argument('--p', type=float, default=1,
                        help='Return hyperparameter. Default is 1.')

    parser.add_argument('--q', type=float, default=1,
                        help='Inout hyperparameter. Default is 1.')

    parser.add_argument('--weighted', dest='weighted', action='store_true',
                        help='Boolean specifying (un)weighted. Default is unweighted.')
    parser.add_argument('--unweighted', dest='unweighted', action='store_false')
    parser.set_defaults(weighted=False)

    parser.add_argument('--directed', dest='directed', action='store_true',
                        help='Graph is (un)directed. Default is undirected.')
    parser.add_argument('--undirected', dest='undirected', action='store_false')
    parser.set_defaults(directed=False)

    return parser.parse_args()


def read_graph():
    """
    Reads the input network in networkx.
    """
    if args.weighted:
        G = nx.read_edgelist(args.input, nodetype=int, data=(('weight', float),), create_using=nx.DiGraph())
    else:
        G = nx.read_edgelist(args.input, nodetype=int, create_using=nx.DiGraph())
        for edge in G.edges():
            G[edge[0]][edge[1]]['weight'] = 1

    if not args.directed:
        G = G.to_undirected()

    return G


def learn_embeddings(walks):
    """
    Learn embeddings by optimizing the Skipgram objective using SGD.
    """
    walks = [map(str, walk) for walk in walks]
    model = Word2Vec(walks, size=args.dimensions, window=args.window_size, min_count=0, sg=1, workers=args.workers,
                     iter=args.iter)
    model.wv.save_word2vec_format(args.output)

    return


def main():
    """
    Pipeline for representational learning for all nodes in a graph.
    """
    nx_G = read_graph()
    G = n2v.Graph(nx_G, args.directed, args.p, args.q)
    G.preprocess_transition_probs()
    walks = G.simulate_walks(args.num_walks, args.walk_length)
    np.savetxt(args.output, walks)


if __name__ == "__main__":
    args = parse_args()
    main()



import numpy as np


def mark_label_on_pathways(name, pid, pw_map, gene_id_list, label=1):
    """Marks given genes to the pathways

    Parameters
    ----------
    name: str
    pid: int
        patient id
    pw_map: map of networkx graphs of pathways
        patient label mapping
    gene_id_list: list of list of string
        uniprot gene id list of genes
    label: int
        the label which will be assigned to found genes in pathways - default value is 1
    """
    label_field = f'label-{name}'
    gene_ids = [uid for a in gene_id_list for uid in a]
    for pw in pw_map.values():  # for each pathway
        for n in pw.nodes():
            nd = pw.nodes[n]
            if label_field not in nd:
                pw.add_node(n, **{label_field: {}})
            if np.any([g in nd['uniprotids'] for g in gene_ids]):
                nd[label_field][pid] = label


def mark_cont_label_on_pathways(name, pid, pw_map, uni_ids, gene_vals):
    """Marks given genes and their normalized expressions to the pathways

    Parameters
    ----------
    name: str
    pid: int
            patient id
    pw_map: map of networkx graphs of pathways
            patient label mapping
    uni_ids: list of list of string
            uniprot gene id list of genes
    gene_vals: :obj:`numpy.ndarray`
            the values of genes which will be assigned to found genes in pathways
    """
    label_field = f'label-{name}'
    # gene_ids = uni_ids #[uid for a in gene_id_list for uid in a]
    for pw in pw_map.values():  # for each pathway
        for n in pw.nodes():
            nd = pw.nodes[n]
            if label_field not in nd:
                pw.add_node(n, **{label_field: {}})
            intersect_values = gene_vals[[len(set(nd['uniprotids']).intersection(g)) > 0 for g in uni_ids]]
            if len(intersect_values) > 0:
                if 'oe' in name:
                    nd[label_field][pid] = max(0, max(intersect_values))
                elif 'ue' in name:
                    nd[label_field][pid] = min(0, min(intersect_values))
                elif 'abs' in name:
                    nd[label_field][pid] = max(intersect_values.max(), intersect_values.min(), key=abs)


def mark_extra_label_on_pathways(name, pid, pw_map, old_label_name, threshold=1.96):
    """Marks new labels on pathways using old_labels

    Parameters
    ----------
    name: string
        new label name
    pid: int
        patient id
    pw_map: map of networkx graphs of pathways
        patient label mapping
    old_label_name: string
        old label name that will be used for new label
    threshold: float
        threshold of new label. If abs(old_label)<threshold then new label=0 same otherwise - default value is 1.96
    """
    label_field = f'label-{name}'
    old_label_field = f'label-{old_label_name}'
    oe_label_field = f'label-oe'
    ue_label_field = f'label-ue'
    for pw in pw_map.values():  # for each pathway
        for n in pw.nodes():
            nd = pw.nodes[n]
            if label_field not in nd:
                pw.add_node(n, **{label_field: {}})
            if name == 'onekernel':
                if pid in nd[oe_label_field].keys():
                    nd[label_field][pid] = nd[oe_label_field][pid]
                elif pid in nd[ue_label_field].keys():
                    nd[label_field][pid] = nd[ue_label_field][pid]
            else:
                if pid in nd[old_label_field].keys() and abs(nd[old_label_field][pid]) < threshold:
                    nd[label_field][pid] = 0
                elif pid in nd[old_label_field].keys():
                    nd[label_field][pid] = nd[old_label_field][pid]



def pandas_to_latex_table(df, row_name, col_name, file_path):
    col_headers = list(df.columns)
    row_headers = list(df.index)
    col_size = len(col_headers)

    # begin
    table_str_full = '\\begin{table}[H]\n\\centering\\captionsetup{width=.75\\textwidth}\n'

    # col setup
    table_str_full += '\\begin{tabular}{|p{2cm}|' + 'p{2cm}|' * col_size + '}\n\\hline\n'
    table_str_full += f'\\textbf{{${row_name}$/${col_name}$}}' + ' & \\textbf{0}' * len(col_headers) + ' \\\\ \\hline\n'

    # rows
    for row_header in row_headers:
        table_str_row_value = ' & '.join(str(df[col_header][row_header]) for col_header in col_headers)
        table_str_full += '\\textbf{0} & ' + table_str_row_value + ' \\\\ \\hline \n'

    # end
    table_str_full += '\\end{tabular}\n\\caption{}\n\\label{tab:}\n\\end{table}\n'

    with open(file_path, 'w') as out_file:
        out_file.write(table_str_full)
    return table_str_full



"""
    This file is to process clincal file
    It is enough to call processOneClinicalSomatic with file location as argument to function:
        import synapse_som_processor as ssp
        ssp.processOneCancerSomatic('folder/fileName.maf')
        This returns 2d numpy array with columns PatientId, AliveOrDead(0 or 1),
         days_to_last_followup or days_to_death(alive or dead), Stage

    To write the report to a file:
        import processors.synapse_clinical_processor as clinic
        output = clinical.processOneClinicalSomatic('folder/fileName.maf')
        ssp.writeToFile(output, fileLocation)



"""

import csv

import numpy as np

from .. import config

CANCER_TYPES = ['BLCA', 'BRCA', 'COAD', 'GBM', 'HNSC', 'KIRC', 'LAML', 'LUAD', 'LUSC', 'OV', 'READ', 'UCEC']


def process_one_clinical_somatic(filepath, delimiter='\t', start_row=1):
    data_array = []

    with open(filepath, 'r', encoding='utf8', errors='ignore') as file:
        csv_reader = csv.reader(file, delimiter=delimiter)
        for i in range(start_row):
            header = next(csv_reader)
        for row in csv_reader:
            data_array.append(row)

    if 'tumor_stage' in header:
        ts_key = 'tumor_stage'
    elif 'ajcc_neoplasm_disease_stage' in header:
        ts_key = 'ajcc_neoplasm_disease_stage'
    elif 'gynecologic_tumor_grouping_figo_stage' in header:
        ts_key = 'gynecologic_tumor_grouping_figo_stage'
    else:
        ts_key = '#'
    data_indexes = [header.index('#'), header.index('vital_status'), header.index('days_to_death'),
                    header.index('days_to_last_followup'), header.index(ts_key)]

    '''
    for idx, col in enumerate(header):
        if col == '#':
            data_indexes[0] = idx
        if col == 'tumor_stage':
            data_indexes[4] = idx
        if col == 'vital_status':
            data_indexes[1] = idx
        if col == 'days_to_death':
            data_indexes[2] = idx
        if col == 'days_to_last_followup':
            data_indexes[3] = idx

    '''

    data_array_processed = []
    for row in data_array:
        tmp = row[data_indexes[0]]
        splitted = tmp.split('-')
        if (len(splitted) > 3) and ('01' in splitted[3]):
            new_val = '-'
            new_val = new_val.join(splitted[0:3])
            if row[data_indexes[1]] == 'LIVING':
                temp_days = row[data_indexes[3]]
                row3_value = 0
            else:
                temp_days = row[data_indexes[2]]
                row3_value = 1
            stage = 'NA'
            if row[data_indexes[4]] != 'NA':
                checker = row[data_indexes[4]]
                if ('iv' in checker) or ('IV' in checker):
                    stage = 4
                elif ('iii' in checker) or ('III' in checker):
                    stage = 3
                elif ('ii' in checker) or ('II' in checker):
                    stage = 2
                elif ('i' in checker) or ('I' in checker):
                    stage = 1
                '''
                listFrom = ['Stage I', 'Stage II', 'Stage III', 'Stage IV']
                if row[data_indexes[4]] in listFrom:
                    indforTo = listFrom.index(row[data_indexes[4]])
                    listTo = [1, 2, 3, 4]
                    stage = listTo[indforTo]
                '''
            temp_list = [new_val] + [row3_value] + [temp_days] + [stage]
            data_array_processed.append(temp_list)

    return np.array(data_array_processed)


# 'kirc/data/synapse_kirc_som_data.csv'
def write_to_file(report, filepath):
    with open(filepath, 'w', newline='') as csv_file:
        csv_writer = csv.writer(csv_file, delimiter=',', quotechar='|', quoting=csv.QUOTE_MINIMAL)
        csv_writer.writerow(['Patient ID', 'Status', 'Days', 'Stage'])
        csv_writer.writerows(report)


def print_report(report):
    if len(report) != 0:
        print(len(report))
        print(len(list(set(report[:, 0]))))
        alive_count = np.count_nonzero(report[:, 1] == '0')
        dead_count = np.count_nonzero(report[:, 1] == '1')
        stage_counts = [np.count_nonzero(report[:, 3] == str(i + 1)) for i in range(4)]
        print(alive_count)
        print(dead_count)
        for s in stage_counts: print(s)


def report_all_cancer_types(dir_path):
    for ct in CANCER_TYPES:
        filepath = dir_path / ct / 'clinical'
        report = process_one_clinical_somatic(filepath)
        print(ct + '# of row, # of Unique Gene, #of Unique Patient')
        print_report(report)


CANCER_TYPE = 'OV'

FILEPATH = config.ROOT_DIR / CANCER_TYPE / 'clinical'
rep = process_one_clinical_somatic(FILEPATH)
write_to_file(rep, config.DATA_DIR / 'ov_data' / 'ov_clinical_data.csv')

# report_all_cancer_types(DATA_DIR)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import networkx as nx

from . import kgml_pathway_reader
from .. import config
from ..lib.sutils import *


@timeit
def KGML_to_networkx_graph(pathway_id, is_directed, entries=None, relations=None):
    """
    @param is_directed forced to prevent parsing the graph wrongly
    """
    print('Converting KGML pathway to networkx pathway:', pathway_id)
    if entries is None or relations is None:
        entries, relations = kgml_pathway_reader.get_pathway_kgml(pathway_id)

    # validate data dir
    safe_create_dir(config.DATA_DIR)
    # convert kgml data to networkx graph
    nodes = [(eid, {'name': entries[eid].name, 'hname': entries[eid].graphics[0].name, 'eid': eid }) for eid in entries]
    edges = [(r.entry1.id, r.entry2.id, {'weight': 1}) for r in relations]
    nx_G = nx.Graph()
    nx_G.add_nodes_from(nodes)
    nx_G.add_edges_from(edges)
    # remove 0 degree info nodes
    # nx_g.remove_nodes_from([n for n in nx.isolates(nx_g)])
    print('Nodes with no edges:', len([n for n in nx.isolates(nx_G)]))
    path = config.DATA_DIR / f'{pathway_id}.gml'
    print('Saving GML to path:', path)
    nx.write_gml(nx_G, path)

    print('Finished conversion to networkx graph pathway:', pathway_id)
    if is_directed:
        return nx_G, entries, relations
    else:
        return nx_G.to_undirected(), entries, relations



__all__ = [
    'center_product_kernel',
    'lmkkmeans_train',
    'main',
    'neighbor_walk_kernel',
    'new_kernel',
    'node2vec_h_i',
    'node2vec_h_i_k',
    'pairwise_product_max_kernel',
    'pamogk',
]



import numpy as np
from sklearn.svm import SVC


def calculate_s_and_max_sd(patients, gene_vectors):
    # calculate S (mutated gene vector set) and P (average mutataion point) vector
    for p in patients:
        genes = np.array([gene_vectors[str(n)] for n in p['mutated_nodes']])
        p['S'] = genes


def max_sd_kernel(p1, p2):
    # calculate maximum S difference for given pair
    max_d = None
    for g1 in p1['S']:
        for g2 in p2['S']:
            d = np.dot(g1, g2)
            if max_d is None or max_d < d:
                max_d = d
    return max_d


def test_accr(patients):
    hit = 0
    pids = [p['pid'] for p in patients]
    for pid in pids:
        test_p = [p for p in patients if p['pid'] == pid][0]
        train_p = [p for p in patients if p['pid'] != pid]

        linear_svc = SVC(kernel='linear')
        linear_svc.fit([p['P'] for p in train_p], [p['sick'] for p in train_p])
        is_hit = linear_svc.predict([test_p['P']]) == [test_p['sick']]
        # print('%3d: %s' % (pid, is_hit), linear_svc.predict([test_p['P']]), test_p['pid'], test_p['sick'])
        hit += is_hit[0]
    print('Accuracy Leave-One-Out accuracy=%.2lf' % (hit / len(pids)))

    hit = 0
    pids = [p['pid'] for p in patients]
    lpid = len(pids)
    K = 10
    S = np.math.ceil(lpid / K)
    for i in range(0, lpid, S):
        test_p = patients[i:i + S]
        train_p = patients[:i] + patients[i + S:]

        linear_svc = SVC(kernel='linear')
        linear_svc.fit([p['P'] for p in train_p], [p['sick'] for p in train_p])
        is_hit = linear_svc.predict([p['P'] for p in test_p]) == [p['sick'] for p in test_p]
        # print('%3d: %s' % (i, is_hit))
        hit += np.sum(is_hit)
    print('Accuracy K-fold with K=%d accuracy=%.2lf' % (K, hit / len(pids)))



#!/usr/bin/env python3
import json
import pdb

import numpy as np
import requests

from .. import config
from ..pathway_reader import cx_pathway_reader

UNIPROT_DOMAIN = 'www.uniprot.org'
UNIPROT_HOST = f'https://{UNIPROT_DOMAIN}'


def get_all_nodes(all_pws_map):
    return np.hstack([[g.nodes[n] for n in g.nodes()] for g in all_pws_map.values()])


def get_alias_set(all_pws_map):
    all_nodes = get_all_nodes(all_pws_map)
    return np.unique(np.hstack([n['alias'] for n in all_nodes if 'alias' in n]))


def get_non_alias_set(all_pws_map):
    all_nodes = get_all_nodes(all_pws_map)
    return np.unique(np.hstack([n['n'] for n in all_nodes if 'alias' not in n]))


def fetch_uniprot_mapping(alias_list, fr='ACC,ID', to='P_ENTREZGENEID'):
    print('Requesting mapping for', len(alias_list), 'genes')
    alias_list_str = ",".join(alias_list)
    path = config.DATA_DIR / 'all_aliases.txt'
    with open(path, 'w') as f:
        f.write(alias_list_str)
    url = f'{UNIPROT_HOST}/uploadlists/'
    r = requests.post(url,
                      headers={
                          'authority': UNIPROT_DOMAIN,
                          'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
                          'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
                          'upgrade-insecure-requests': '1',
                          'referer': url,
                      },
                      data={
                          'landingPage': 'false',
                          'jobId': '',
                          'uploadQuery': alias_list_str,
                          'url2': '',
                          'from': fr,
                          'to': to,
                          'taxon': '',
                      })
    mapping_id = None
    if len(r.history) > 0:
        mapping_id = r.history[0].headers['location'].split('/')[-1]

    if r.status_code != requests.codes.ok:
        raise Exception(f'Failed to request mapping data with status={r.status_code} and reason={r.text}')

    if mapping_id is None:
        raise Exception('Could not find mapping for aliases')

    print('Received mapping with id:', mapping_id)

    r = requests.get('%s/mapping/%s.tab' % (UNIPROT_HOST, mapping_id))
    if r.status_code != requests.codes.ok:
        raise Exception(f'Failed to get mapping data with status={r.status_code} and reason={r.text}')

    rows = [r.split('\t') for r in r.text.split('\n')[1:-1]]
    mapped = [{'from': r[0], 'to': r[1]} for r in rows]

    r = requests.get('%s/mapping/%s.not' % (UNIPROT_HOST, mapping_id))
    if r.status_code != requests.codes.ok:
        raise Exception(f'Failed to get missing data with status={r.status_code} and reason={r.text}')

    missing = [r for r in r.text.split('\n')[1:-1]]

    return mapped, missing


def fetch_uniprot_to_entrez(alias_list):
    uniprot_alias_list = [a[1] for a in [a.split('uniprot knowledgebase:') for a in alias_list] if len(a) > 1]
    return fetch_uniprot_mapping(uniprot_alias_list)


def get_uniprot_to_entrez_map(all_pws_map=None):
    if all_pws_map is None:
        all_pws_map = cx_pathway_reader.read_pathways()

    all_alias = get_alias_set(all_pws_map)

    mapped, missing = fetch_uniprot_to_entrez(all_alias)

    json_data = {
        'mapped': mapped,
        'missing': missing,
    }

    path = config.DATA_DIR / 'gene-map-uniprot-entrez.json'
    with open(path, 'w') as f:
        json.dump(json_data, f)


def json_to_dict():
    path = config.DATA_DIR / 'gene-map-uniprot-entrez.json'
    with open(path, 'r') as f:
        data = json.load(f)

    mapped_data = data['mapped']
    missing_data = data['missing']

    uni_prot_to_entrez = {}
    entrez_to_uni_prot = {}

    for line in mapped_data:
        safe_list_value_append(uni_prot_to_entrez, line['from'], line['to'])
        safe_list_value_append(entrez_to_uni_prot, line['to'], line['from'])

    for gene in missing_data:
        if gene not in uni_prot_to_entrez:
            uni_prot_to_entrez[gene] = []

    return uni_prot_to_entrez, entrez_to_uni_prot


def safe_list_value_append(d, k, v):
    if k not in d:
        d[k] = [v]
    else:
        d[k].append(v)


if __name__ == '__main__':
    get_uniprot_to_entrez_map()
    json_to_dict()
    pdb.set_trace()



__all__ = [
    'kirc_to_rmkl',
    'kirc_to_vec',
    'node2vec_processor',
    'paradigm_data_prep',
    'rnaseq_processor',
    'synapse_clinical_processor',
    'synapse_rppa_processor',
    'synapse_som_processor',
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import csv
from pathlib import Path

from .. import config
from ..lib.sutils import ensure_file_dir

KIRC_DATA_DIR = config.DATA_DIR / 'kirc_data'

parser = argparse.ArgumentParser(description='Run SPK algorithms on pathways')
parser.add_argument('--somatic-data', '-r', metavar='file-path', dest='somatic_data', type=str2path, help='Somatic Data',
                    default=KIRC_DATA_DIR / 'kirc_somatic_mutation_data.csv')


def read_csv(filename, delimiter=','):
    with open(KIRC_DATA_DIR / filename, encoding='utf8') as f:
        return [r for r in csv.DictReader(f, delimiter=delimiter)]


def transpose_data(data, id_col, ignored_cols=None):
    """
    transposes given data from patient columns to patient rows
    data: list
    id_col: str
    ignored_cols: list
    :return: list
    """
    if ignored_cols is None:
        ignored_cols = []
    ignored_cols.append(id_col)  # add id col to ignores
    tm = {}
    pat_ids = set(data[0].keys()).difference(ignored_cols)
    for p in pat_ids:  # initialize patients
        tm[p] = {'Patient ID': p}
    for r in data:
        col_id = r[id_col]
        for p in pat_ids:
            tm[p][col_id] = r[p]
    return list(tm.values())


def add_data_to_patient(key, data, pat_data=None):
    """
    adds kernel data to data map

    :param key:
    :param data:
    :param pat_data:
    :return:
    """
    if pat_data is None:
        pat_data = {}
    for r in data:
        # patient ids for rppa have some extra fields
        pat_id = r['Patient ID'] = '-'.join(r['Patient ID'].split('-')[:3])
        if pat_id not in pat_data:
            pat_data[pat_id] = {}
        pat_data[pat_id][key] = r
    return pat_data


def write_csv(data, key, pat_ids, use_index=False):
    filepath = KIRC_DATA_DIR / 'rmkl' / f'{key}.csv'
    print('Writing patient data type', key, 'to path', filepath)
    ensure_file_dir(filepath)
    with open(filepath, 'w') as f:
        key_data = [r[key] for r in data]
        if use_index:
            for r in key_data:
                r['Patient ID'] = pat_ids.index(r['Patient ID'])
        # make sure patient id is the first column
        fieldnames = ['Patient ID'] + list(k for k in key_data[0].keys() if k not in ['Patient ID'])
        csv_writer = csv.DictWriter(f, fieldnames)
        csv_writer.writeheader()
        csv_writer.writerows(key_data)


def write_pat_id_map(full_pat_ids):
    filepath = KIRC_DATA_DIR / 'rmkl' / 'pat-id-map.csv'
    print('Writing patient id map to path', filepath)
    ensure_file_dir(filepath)
    with open(filepath, 'w') as f:
        csv_writer = csv.DictWriter(f, ['Patient ID', 'index'])
        csv_writer.writeheader()
        for i, pat_id in enumerate(full_pat_ids):
            csv_writer.writerow({'Patient ID': pat_id, 'index': i})


def main():
    clinical_data = read_csv('kirc_clinical_data.csv')
    # somatic_mut_data = read_csv('kirc_somatic_mutation_data.csv')
    rnaseq_data = transpose_data(read_csv('kirc_rna_seq_expression_data.csv'), 'Entrez Gene ID', ['Gene Name'])
    rppa_data = transpose_data(read_csv('kirc_rppa_data', delimiter='\t'), '#probe')

    data = add_data_to_patient('clinical', clinical_data)
    # data = add_data_to_patient('somatic', somatic_mut_data, data)
    data = add_data_to_patient('rnaseq', rnaseq_data, data)
    data = add_data_to_patient('rppa', rppa_data, data)

    print('Total patients:', len(data))

    # get patients that have all 4 data
    full_pats_ids = [k for k, v in data.items() if len(v) == 3]
    full_pats = [data[pat_id] for pat_id in full_pats_ids]
    print('Patients with full data:', len(full_pats))

    write_pat_id_map(full_pats_ids)
    write_csv(full_pats, 'clinical', full_pats_ids)
    # write_csv(full_pats, 'somatic', full_pats_ids)
    write_csv(full_pats, 'rnaseq', full_pats_ids)
    write_csv(full_pats, 'rppa', full_pats_ids)

    # illimuna_data = read_csv('unc.edu_KIRC_IlluminaHiSeq_RNASeqV2.geneExp.whitelist_tumor.txt')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__all__ = [
    'config',
    'label_mapper',
    'lib',
    'data_processor',
    'gene_mapper',
    'kernels',
    'pathway_reader',
]

__version__ = '2.2.0'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys

import requests
from Bio.KEGG.KGML import KGML_parser
from Bio.KEGG.KGML.KGML_pathway import Relation

from .. import config
from ..lib.sutils import *

HOST = 'http://rest.kegg.jp'

DATA_ROOT = config.DATA_DIR / 'kegg'

safe_create_dir(DATA_ROOT)


@timeit
def get_all_pathways(force=False):
    list_url = f'{HOST}/list/pathway/hsa'
    path = DATA_ROOT / 'kegg-all-pathways.txt'
    if force or not path.exists():
        r = requests.get(list_url)
        if not r.status_code == 200:
            print('Failed to get pathway:', path, r.text)
            sys.exit(-1)
        with open(path, 'w') as f:
            f.write(r.text)
            print('Saved KGML file at:', path)

    with open(path) as f:
        lines = [line for line in f.readlines() if line.strip() != '']
        all_pathways = [line.split('\t')[0] for line in lines]
        print('Number of pathways:', len(all_pathways))
        return all_pathways


def get_pathway_kgml(pathway_id='hsa04151'):
    print('Reading pathway:', pathway_id)
    pw_url = f'{HOST}/get/{pathway_id}/kgml'

    # get pathway data if not exists
    path = DATA_ROOT / f'{pathway_id}.kgml'
    if not path.exists():
        print('Could not find hsa file getting from kegg:', pw_url)
        r = requests.get(pw_url)
        if not r.status_code == 200:
            print('Failed to get pathway:', path, r.text)
            sys.exit(-1)
        with open(path, 'w') as f:
            f.write(r.text)
            print('Saved KGML file at:', path)

    # parse data
    with open(path, 'r') as f:
        pathway = KGML_parser.read(f)
    print('\n', pathway, sep='')
    print('  entry:', len(pathway.entries))
    print('  reaction:', len(pathway.reactions))
    print('  relation:', len(pathway.relations))
    entries, relations = prune_kgml(pathway)
    print('Finished reading:', pathway_id)
    print(' entry:', len(entries.keys()), 'relation:', len(relations), 'new_relation:',
          len([1 for r in relations if hasattr(r, '_pamogk')]))
    return entries, relations


def prune_kgml(pathway):
    """
    prune_KGML prune given kgml pathway
      This function prunes the given entries and relations by deleting
      empty entries and corresponding relations, and empty relations and
      corresponding entries. Further pruning is done for our case. We delete
      inner pathways, complex components, PCrel (by inserting edge among
      nodes it connects), maplink, orthogonals
      Inputs:
        pathway:
            entries: index map of entries
            relations: list of relations
      Outputs:
          entries: pruned entries
          relations: pruned relations
          new_relations: new relations added in place of removed compounds
    """

    # copy relations as list and entries as dict
    entries = dict([(e.id, e) for e in pathway.entries.values()])
    relations = [r for r in pathway.relations]
    new_relations = []

    # replace groups with their components and relations between components
    for e in [e for e in entries.values() if e.type == 'group']:
        comp_eids = [comp.id for comp in list(e.components)]
        for e1_id in comp_eids:
            for e2_id in comp_eids:
                if can_add_relation(e1_id, e2_id, relations, new_relations):
                    new_relations.append((e1_id, e2_id))

    # prune orthologs, groups, other, map in entries
    # if any "other" type of entry is included, relations that has
    # these entries are removed from the relations
    type_filter = ['ortholog', 'group', 'other', 'map']
    relations = [r for r in relations if r.entry1.type not in type_filter and r.entry2.type not in type_filter]
    # remove orthologs, groups, and others from entries
    for e in [e for e in entries.values() if e.type in type_filter]:
        del entries[e.id]

    # % prune compounds in entries
    # retrieve gene ids
    gene_ids = set([e.id for e in entries.values() if e.type == 'gene'])

    # for every compound
    for cid in [e.id for e in entries.values() if e.type == 'compound']:
        # find indexes of compounds in relations as source and
        # derive the related ids from it
        source_ids = set([r.entry1.id for r in relations if r.entry2.id == cid])
        source_set = source_ids & gene_ids  # take just genes, ignore others

        # find indexes of compounds in relations as destination
        # and derive the related ids from it
        dest_ids = set([r.entry2.id for r in relations if r.entry1.id == cid])
        dest_set = dest_ids & gene_ids  # take just genes, ignore others

        # create the relation among source and destination if
        # it is not self-relation and if the relation does not already
        # exist in the relations
        # counter = 0
        if not source_ids or not dest_ids:
            continue
        print('src =', source_ids, 'dst =', dest_ids)
        for tmp_s in source_set:
            for tmp_d in dest_set:
                # check if the relation already exists
                if can_add_relation(tmp_s, tmp_d, relations, new_relations):
                    new_relations.append((tmp_s, tmp_d))
                else:
                    print([tmp_s, tmp_d], 'already in relations')
        # removed relations containing this compound entry
        relations = [r for r in relations if r.entry1.id != cid and r.entry2.id != cid]

        # remove compounds from entries
        del entries[cid]

    # turn new relations to relation objects
    new_rel = []
    for s, d in new_relations:
        r = Relation()
        r._entry1 = entries[s]
        r._entry2 = entries[d]
        r._pamogk = True  # to mark relation as added by PAMOGK
        new_rel.append(r)
    relations += new_rel

    return entries, relations


def can_add_relation(s, d, relations, new_relations):
    """
    returns true if (s, d) is a valid new relation
    """
    if s == d:
        return False
    for r in relations:
        if r.entry1.id == s and r.entry2.id == d:
            return False
        if r.entry2.id == s and r.entry1.id == d:
            return False
    for ns, nd in new_relations:
        if ns == s and nd == d:
            return False
        if nd == s and ns == d:
            return False
    return True


if __name__ == '__main__':
    for pw_id in get_all_pathways():
        print(pw_id)
        entry_map, rels = get_pathway_kgml(pw_id)
        print(pw_id, len(entry_map.keys()), len(rels), '\n')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pamogk',
 'pamogk.data_processor',
 'pamogk.gene_mapper',
 'pamogk.kernels',
 'pamogk.lib',
 'pamogk.pathway_reader',
 'pamogk.result_processor']

package_data = \
{'': ['*']}

install_requires = \
['biopython>=1.76',
 'gensim>=3.8.0',
 'mkkm_mr>=0.2.1,<0.3.0',
 'mosek>=9.2.0',
 'networkx>=2.4.0',
 'numpy>=1.18.0',
 'pandas>=1.0.0',
 'requests>=2.23.0',
 'scikit-learn>=0.23.0',
 'scipy>=1.5.2,<2.0.0',
 'snf_simple>=0.1.0,<0.2.0',
 'snfpy']

setup_kwargs = {
    'name': 'pamogk',
    'version': '2.2.0',
    'description': 'PAMOGK',
    'long_description': "PAMOGK\n=====\n\n# Installation\n\n## Getting the Code Base\nYou can either checkout the code base through git:\n```bash\ngit clone https://github.com/tastanlab/pamogk.git\n```\nor you can download the master branch from:\n> https://github.com/tastanlab/pamogk/archive/master.zip\n\n## Python Environment\nProject mainly uses python, but some of the plot generation codes use matlab or output html files (plotly.js). Also some\n libraries might be dependent on operating system. In our tests we used Debian 12.04LTS and macOS Catalina.\n\nFirst install `pipenv` using the instructions given:\n\n> https://github.com/pypa/pipenv#installation\n\nWe are using [pipenv](https://github.com/pypa/pipenv#installation) because of version locking and predictive builds.\n\n## Setting up MOSEK\nWe are using [MOSEK Optimizer API](https://docs.mosek.com/9.1/pythonapi/index.html) for optimizing view weights, and in\norder to use MOSEK you will need a license file. MOSEK provides free certificates for academic purposes and free trial\ncertificates for Commercial usages. You can see check the link below for information on how to acquire an academic license:\n\n> https://www.mosek.com/products/academic-licenses/\n\n### License File Placement\nYou can either place the license file under user's home directory in a folder named `mosek` e.g:\n```bash\n$HOME/mosek/mosec.lic # *nix/macOS users\n%USERPROFILE%\\mosek\\mosek.lic # windows users\n```\nOther option is to set the `MOSEKLM_LICENSE_FILE` environment variable to your license file location.\n\nWe also look for the license file paths in some custom paths (by overwriting `MOSEKLM_LICENSE_FILE` if not present).\nYou can check `pamogk.config.MOSEK_SUPPORTED_PATHS` to see a list of supported paths.\n\nFor more information:\n\n> https://docs.mosek.com/9.2/install/installation.html#setting-up-the-license\n\n## Dependencies\nInstall dependencies with:\n```bash\npipenv install\n```\n\n## Required Environment Variables\nYou can add a `.env` file that will be loaded by `pipenv`\n\nAlso please don't forget to export `MOSEKLM_LICENSE_FILE` variable as well if you are using a path other\nthan `$HOME/mosek/mosek.lic`:\n\n```bash\n# required for experiment running\nPYTHONPATH=${PYTHONPATH}:.\nMOSEKLM_LICENSE_FILE=$MY_MOSEK_LICENSE_FILE_PATH\n```\n\n# Folder Structure\n- **pamogk:** package root folder\n  - **kernels:** Kernel methods to calculate distance between patients. Will try to extend these!\n  - **data_processor:** Data processors/converters used to convert different forms of data to usable forms.\n   Will try to extend these!\n  - **gene_mapper:** Uniprot/Entrez gene mapping matcher for different datasets using different gene IDs. Will try to\n   separate to its own package or along with other tools.\n  - **pathway_reader:** https://.ndexbio.org CX format, and http://kegg.jp KGML format reader and converters. Will try\n   to separate to its own package or along with other tools.\n- **data:** Data files that are both raw or generated by the project. Keeping intermediate files saves a lot of\n computation time where applicable. (not included in package)\n- **experiments:** Experiments done on real data. (not included in package)\n\n\n# Experiments\n## Running Experiments\nYou should run experiments either by starting a pipenv environment shell by:\n```bash\npipenv shell\n```\n\nor through pipenv with:\n```bash\npipenv run experiments/pamogk_all_exp_1.py\n```\nThis ensures that experiments are run with correct environment.\n\n## Experiment Parameters\nExperiments are code entry points that have arguments that change per experiment. You can see their arguments by\n running them with `--help` parameter. e.g:\n```bash\n./experiments/pamogk_all_exp_1.py --help\n```\nand you should see help as such:\n```\nusage: pamogk_all_exp_1.py [-h] [--rs-patient-data file-path]\n                           [--rp-patient-data file-path]\n                           [--som-patient-data file-path]\n\nRun PAMOGK-mut algorithms on pathways\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --rs-patient-data file-path, -rs file-path\n                        rnaseq pathway ID list\n  --rp-patient-data file-path, -rp file-path\n                        rppa pathway ID list\n  --som-patient-data file-path, -s file-path\n                        som mut pathway ID list\n```\n",
    'author': 'Fma',
    'author_email': 'fmakdemir@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tastanlab/pamogk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os

from .lib.sutils import *

ROOT_DIR = os.environ.get('PAMOGK_ROOT_DIR', None)
if ROOT_DIR is not None:
    ROOT_DIR = Path(ROOT_DIR)

if ROOT_DIR is None or not ROOT_DIR.exists() or not ROOT_DIR.is_dir():
    log('Invalid PAMOGK_ROOT_DIR environment variable. Please set PAMOGK_ROOT_DIR to root of data and result folder')
    ROOT_DIR = Path(__file__).resolve().parent.parent
    log('set PAMOGK_ROOT_DIR to ', ROOT_DIR)

log(f'PAMOGK_ROOT_DIR={ROOT_DIR}')
DATA_DIR = ROOT_DIR / 'data'

# if mosek env var is not given check custom valid paths
_MOSEK_LIC_FILE_ENV = 'MOSEKLM_LICENSE_FILE'
MOSEK_SUPPORTED_PATHS = ['~/mosek/mosek.lic', '~/.mosek/mosek.lic', '~/.mosek.lic']
if _MOSEK_LIC_FILE_ENV not in os.environ:
    for p in MOSEK_SUPPORTED_PATHS:
        _p = Path(p).expanduser()
        if _p.exists():
            _p = str(_p)
            os.environ[_MOSEK_LIC_FILE_ENV] = _p
            os.putenv(_MOSEK_LIC_FILE_ENV, _p)
            log(f'{_MOSEK_LIC_FILE_ENV} not found but mosek file on another path:', p)
            break
# store for logging
MOSEK_LICENCE_FILE_PATH = os.getenv(_MOSEK_LIC_FILE_ENV)

safe_create_dir(DATA_DIR)


def get_safe_data_file(file_path: Path):
    """

    Parameters
    ----------
    file_path: :obj:`pathlib.Path`
    Data file path, if it is absolute will use as is, otherwise will look under data folder

    Returns
    -------
    normalized data file path

    """
    file_path = get_safe_path_obj(file_path)
    if file_path.is_absolute():
        return file_path
    return DATA_DIR / file_path



from gensim.models import Word2Vec

from ..lib import node2vec

'''
TODO: migrate to https://github.com/eliorc/node2vec
'''


def process(nx_g, args, gene_vec_conv=lambda x: x):
    """Generates node2vec representation of genes for given pathway network
    Parameters
    ----------
    nx_g: :networkx.classes.graph.Graph:
        pathway graph
    args: Namespace
        node2vec arguments see node2vec
    gene_vec_conv: function
        gene vector converter function
    """
    # generate model
    for v1, v2 in nx_g.edges(): nx_g[v1][v2]['weight'] = 1
    G = node2vec.Graph(nx_g, is_directed=args.is_directed, p=args.p, q=args.q)
    G.preprocess_transition_probs()
    walks_sim = G.simulate_walks(num_walks=10, walk_length=80)
    walks = [list(map(str, walk)) for walk in walks_sim]
    # size=dimension, window=context_size, workers=num_of_parallel_workers, iter=num_epochs_in_sgd
    model = Word2Vec(walks, size=args.n2v_size, window=10, min_count=0, sg=1, workers=4, iter=1)

    # wv has:
    #   * index2entity: array of result index to node id map size N
    #   * vectors: array of result vectors size N x v2v_size
    # so convert them to map G (gene vector map)
    gene_vectors = {}
    for (eid, gene_vec) in zip(model.wv.index2entity, model.wv.vectors):
        gene_vectors[int(eid)] = gene_vec_conv(gene_vec)

    return gene_vectors
